@@ -1,6 +1,6 @@
 /************************************************************************
  * AIPlayer.c     Code for dopewars computer players                    *
- * Copyright (C)  1998-2002  Ben Webb                                   *
+ * Copyright (C)  1998-2003  Ben Webb                                   *
  *                Email: ben@bellatrix.pcl.ox.ac.uk                     *
  *                WWW: http://dopewars.sourceforge.net/                 *
  *                                                                      *
@@ -29,6 +29,7 @@
 #include <sys/types.h>
 #include <errno.h>
 #include <glib.h>
+#include "configfile.h"
 #include "dopewars.h"
 #include "message.h"
 #include "nls.h"
@@ -131,7 +132,7 @@ static void NetBufAuth(NetworkBuffer *netbuf, gpointer data)
  * Main loop for AI players. Connects to server, plays game,
  * and then disconnects.
  */
-void AIPlayerLoop()
+void AIPlayerLoop(struct CMDLINE *cmdline)
 {
   GString *errstr;
   gchar *msg;
@@ -143,6 +144,8 @@ void AIPlayerLoop()
   NBSocksStatus oldsocks;
   NetworkBuffer *netbuf;
 
+  InitConfiguration(cmdline);
+
   errstr = g_string_new("");
   AIPlay = g_new(Player, 1);
 
@@ -157,7 +160,7 @@ void AIPlayerLoop()
   oldstatus = netbuf->status;
   oldsocks = netbuf->sockstat;
 
-  if (!StartNetworkBufferConnect(netbuf, ServerName, Port)) {
+  if (!StartNetworkBufferConnect(netbuf, NULL, ServerName, Port)) {
     AIConnectFailed(netbuf);
     return;
   } else {
@@ -182,7 +185,7 @@ void AIPlayerLoop()
       if (errno == EINTR)
         continue;
       printf("Error in select\n");
-      exit(1);
+      exit(EXIT_FAILURE);
     }
 
     datawaiting =
@@ -334,6 +337,9 @@ int HandleAIMessage(char *Message, Player *AIPlay)
     break;
   case C_PRINTMESSAGE:
     PrintAIMessage(Data);
+    if (AI == C_MISSFIGHT || strncmp(Data, "Too late", 8) == 0) {
+      AIJet(AIPlay);
+    }
     break;
   case C_MSG:
     g_print("%s: %s\n", GetPlayerName(From), Data);
