@@ -1,6 +1,6 @@
 /************************************************************************
  * admin.c        dopewars server administration                        *
- * Copyright (C)  1998-2002  Ben Webb                                   *
+ * Copyright (C)  1998-2003  Ben Webb                                   *
  *                Email: ben@bellatrix.pcl.ox.ac.uk                     *
  *                WWW: http://dopewars.sourceforge.net/                 *
  *                                                                      *
@@ -36,6 +36,7 @@
 #include <stdio.h>
 #include <glib.h>
 
+#include "dopewars.h"
 #include "network.h"
 #include "nls.h"
 #include "serverside.h"
@@ -53,7 +54,7 @@ static int OpenSocket(void)
   sock = socket(PF_UNIX, SOCK_STREAM, 0);
   if (sock == -1) {
     perror("socket");
-    exit(1);
+    exit(EXIT_FAILURE);
   }
 
   addr.sun_family = AF_UNIX;
@@ -63,7 +64,7 @@ static int OpenSocket(void)
   if (connect(sock, (struct sockaddr *)&addr,
               sizeof(struct sockaddr_un)) == -1) {
     perror("connect");
-    exit(1);
+    exit(EXIT_FAILURE);
   }
 
   g_print(_("Connection established; use Ctrl-D to "
@@ -73,7 +74,7 @@ static int OpenSocket(void)
   return sock;
 }
 
-void AdminServer(void)
+void AdminServer(struct CMDLINE *cmdline)
 {
   int sock, topsock;
   NetworkBuffer *netbuf;
@@ -81,6 +82,8 @@ void AdminServer(void)
   gchar *msg, inbuf[200];
   gboolean doneOK;
 
+  InitConfiguration(cmdline);
+
   sock = OpenSocket();
   netbuf = g_new(NetworkBuffer, 1);
 
