@@ -1,6 +1,6 @@
 /************************************************************************
  * curses_client.h  dopewars client using the (n)curses console library *
- * Copyright (C)  1998-2002  Ben Webb                                   *
+ * Copyright (C)  1998-2003  Ben Webb                                   *
  *                Email: ben@bellatrix.pcl.ox.ac.uk                     *
  *                WWW: http://dopewars.sourceforge.net/                 *
  *                                                                      *
@@ -27,6 +27,8 @@
 #include <config.h>
 #endif
 
-void CursesLoop(void);
+struct CMDLINE;
+
+void CursesLoop(struct CMDLINE *cmdline);
 
 #endif
