@@ -1,6 +1,6 @@
 /************************************************************************
  * dopewars.h     Common structures and stuff for dopewars              *
- * Copyright (C)  1998-2002  Ben Webb                                   *
+ * Copyright (C)  1998-2003  Ben Webb                                   *
  *                Email: ben@bellatrix.pcl.ox.ac.uk                     *
  *                WWW: http://dopewars.sourceforge.net/                 *
  *                                                                      *
@@ -43,6 +43,7 @@
 #endif
 
 #include <glib.h>
+#include "convert.h"
 #include "error.h"
 #include "network.h"
 #include "util.h"
@@ -66,6 +67,10 @@ typedef enum {
                                  * notation */
   A_DONEFIGHT,                  /* A fight is only considered over once the
                                  * client sends the server a C_DONE message */
+  A_UTF8,                       /* All strings are sent over the network using
+                                 * UTF-8 (Unicode) encoding */
+  A_DATE,                       /* We can understand "proper" dd-mm-yy dates
+                                 * rather than just turn numbers */
   A_NUM                         /* N.B. Must be last */
 } AbilType;
 
@@ -80,13 +85,15 @@ typedef struct ABILITIES {
 
 struct NAMES {
   gchar *Bitch, *Bitches, *Gun, *Guns, *Drug, *Drugs;
-  gchar *Month, *Year, *LoanSharkName, *BankName;
+  gchar *Date, *LoanSharkName, *BankName;
   gchar *GunShopName, *RoughPubName;
 };
 
 struct SOUNDS {
   gchar *FightHit, *FightMiss, *FightReload, *Jet, *TalkToAll, *TalkPrivate;
-  gchar *JoinGame, *LeaveGame;
+  gchar *JoinGame, *LeaveGame, *StartGame, *EndGame;
+  gchar *EnemyBitchKilled, *BitchKilled, *EnemyKilled, *Killed;
+  gchar *EnemyFailFlee, *FailFlee, *EnemyFlee, *Flee;
 };
 
 #ifdef NETWORKING
@@ -161,23 +168,27 @@ struct LOG {
   FILE *fp;
 };
 
+struct DATE {
+  int day, month, year;
+};
+
+extern gboolean WantAntique;
+extern struct DATE StartDate;
 extern int ClientSock, ListenSock;
-extern gboolean Network, Client, Server, NotifyMetaServer, AIPlayer;
+extern gboolean Network, Client, Server, UseSounds;
 extern unsigned Port;
 extern gboolean Sanitized, ConfigVerbose, DrugValue;
 extern int NumLocation, NumGun, NumCop, NumDrug, NumSubway, NumPlaying,
            NumStoppedTo;
+extern int DebtInterest, BankInterest;
 extern gchar *HiScoreFile, *ServerName, *ConvertFile, *ServerMOTD,
-	     *WantedPlugin;
-extern gboolean WantHelp, WantVersion, WantAntique, WantColour,
-                WantNetwork, WantConvert, WantAdmin;
+	     *BindAddress, *PlayerName;
 #ifdef CYGWIN
 extern gboolean MinToSysTray;
 #else
 extern gboolean Daemonize;
 #endif
 extern gchar *WebBrowser;
-extern ClientType WantedClient;
 extern int LoanSharkLoc, BankLoc, GunShopLoc, RoughPubLoc;
 extern int DrugSortMethod, FightTimeout, IdleTimeout, ConnectTimeout;
 extern int MaxClients, AITurnPause;
@@ -284,6 +295,7 @@ typedef struct TDopeList DopeList;
 struct PLAYER_T {
   guint ID;
   int Turn;
+  GDate *date;
   price_t Cash, Debt, Bank;
   int Health;
   int CoatSize;
@@ -342,6 +354,17 @@ struct GLOBALS {
   int MinVal;
 };
 
+struct CMDLINE { 
+  gboolean help, version, antique, colour, network;
+  gboolean convert, admin, ai, server, notifymeta;
+  gboolean setport;
+  gchar *scorefile, *servername, *pidfile, *logfile, *plugin, *convertfile;
+  gchar *playername;
+  unsigned port;
+  ClientType client;
+  GSList *configs;
+};  
+
 extern const int NUMGLOB;
 extern struct GLOBALS Globals[];
 
@@ -406,13 +429,14 @@ gchar *InitialCaps(gchar *string);
 char StartsWithVowel(char *string);
 char *GetPlayerName(Player *Play);
 void SetPlayerName(Player *Play, char *Name);
-void HandleCmdLine(int argc, char *argv[]);
-void SetupParameters(void);
-void HandleHelpTexts(void);
-void GeneralStartup(int argc, char *argv[]);
+struct CMDLINE *ParseCmdLine(int argc, char *argv[]);
+void FreeCmdLine(struct CMDLINE *cmdline);
+void InitConfiguration(struct CMDLINE *cmdline);
+void HandleHelpTexts(gboolean fullhelp);
+struct CMDLINE *GeneralStartup(int argc, char *argv[]);
 void StripTerminators(gchar *str);
-void ReadConfigFile(char *FileName);
-gboolean ParseNextConfig(GScanner *scanner, gboolean print);
+gboolean ParseNextConfig(GScanner *scanner, Converter *conv,
+                         gchar **encoding, gboolean print);
 int GetGlobalIndex(gchar *ID1, gchar *ID2);
 gchar **GetGlobalString(int GlobalIndex, int StructIndex);
 gint *GetGlobalInt(int GlobalIndex, int StructIndex);
@@ -421,27 +445,28 @@ gboolean *GetGlobalBoolean(int GlobalIndex, int StructIndex);
 gchar ***GetGlobalStringList(int GlobalIndex, int StructIndex);
 void PrintConfigValue(int GlobalIndex, int StructIndex,
                       gboolean IndexGiven, GScanner *scanner);
-gboolean SetConfigValue(int GlobalIndex, int StructIndex,
-                        gboolean IndexGiven, GScanner *scanner);
 gboolean IsCop(Player *Play);
 void RestoreConfig(void);
+void GetDateString(GString *str, Player *Play);
 void ScannerErrorHandler(GScanner *scanner, gchar *msg, gint error);
 gboolean IsConnectedPlayer(Player *play);
 void BackupConfig(void);
+gchar *GetDocRoot(void);
 gchar *GetDocIndex(void);
 gchar *GetGlobalConfigFile(void);
 gchar *GetLocalConfigFile(void);
 
 #ifndef CURSES_CLIENT
-void CursesLoop(void);
+void CursesLoop(struct CMDLINE *cmdline);
 #endif
 
 #ifndef GUI_CLIENT
 #ifdef CYGWIN
 gboolean GtkLoop(HINSTANCE hInstance, HINSTANCE hPrevInstance,
-                 gboolean ReturnOnFail);
+                 struct CMDLINE *cmdline, gboolean ReturnOnFail);
 #else
-gboolean GtkLoop(int *argc, char **argv[], gboolean ReturnOnFail);
+gboolean GtkLoop(int *argc, char **argv[], struct CMDLINE *cmdline,
+                 gboolean ReturnOnFail);
 #endif
 #endif
 
