@@ -1,6 +1,6 @@
 /************************************************************************
  * clist.c        GtkCList implementation for gtkport                   *
- * Copyright (C)  1998-2002  Ben Webb                                   *
+ * Copyright (C)  1998-2003  Ben Webb                                   *
  *                Email: ben@bellatrix.pcl.ox.ac.uk                     *
  *                WWW: http://dopewars.sourceforge.net/                 *
  *                                                                      *
@@ -31,9 +31,15 @@
 #include <windows.h>
 #include <commctrl.h>
 
+#include "unicodewrap.h"
+
 #define LISTITEMHPACK  3
 #define LISTHEADERPACK 6
 
+static const gchar *WC_GTKCLISTHDR = "WC_GTKCLISTHDR";
+
+static WNDPROC wpOrigListProc;
+
 static void gtk_clist_size_request(GtkWidget *widget,
                                    GtkRequisition *requisition);
 static void gtk_clist_set_size(GtkWidget *widget,
@@ -69,11 +75,46 @@ static GtkClass GtkCListClass = {
   gtk_clist_wndproc
 };
 
+static void SetCListHeaderSize(GtkCList *clist)
+{
+  RECT rc;
+  HWND hWnd;
+  int width;
+
+  hWnd = GTK_WIDGET(clist)->hWnd;
+  clist->scrollpos = GetScrollPos(hWnd, SB_HORZ);
+
+  GetWindowRect(hWnd, &rc);
+  width = (int)mySendMessage(hWnd, LB_GETHORIZONTALEXTENT, 0, 0);
+  width = MAX(width, rc.right - rc.left) + 100;
+
+  SetWindowPos(clist->header, HWND_TOP, -clist->scrollpos, 0,
+               width, clist->header_size, SWP_NOZORDER);
+}
+
+static LRESULT APIENTRY ListWndProc(HWND hwnd, UINT msg, WPARAM wParam,
+                                    LPARAM lParam)
+{
+  LRESULT retval;
+  GtkWidget *widget;
+
+  widget = GTK_WIDGET(GetWindowLong(hwnd, GWL_USERDATA));
+  retval = myCallWindowProc(wpOrigListProc, hwnd, msg, wParam, lParam);
+
+  if (msg == WM_HSCROLL && widget) {
+    GtkCList *clist = GTK_CLIST(widget);
+    SetCListHeaderSize(clist);
+  }
+
+  return retval;
+}
+
 gboolean gtk_clist_wndproc(GtkWidget *widget, UINT msg, WPARAM wParam,
                            LPARAM lParam, gboolean *dodef)
 {
   LPDRAWITEMSTRUCT lpdis;
-  HD_NOTIFY FAR *phdr;
+  HD_NOTIFYA FAR *phdr;
+  HD_NOTIFYW FAR *phdrw;
   NMHDR *nmhdr;
 
   switch(msg) {
@@ -87,19 +128,35 @@ gboolean gtk_clist_wndproc(GtkWidget *widget, UINT msg, WPARAM wParam,
     lpdis = (LPDRAWITEMSTRUCT)lParam;
     if (lpdis) {
       gtk_clist_draw_row(GTK_CLIST(widget), lpdis);
-      return FALSE;
+      *dodef = FALSE;
+      return TRUE;
     }
     break;
   case WM_NOTIFY:
     nmhdr = (NMHDR *)lParam;
-    phdr = (HD_NOTIFY FAR *)lParam;
-    if (nmhdr && nmhdr->code == HDN_ENDTRACK) {
-      gtk_clist_set_column_width_full(GTK_CLIST(widget), phdr->iItem,
-                                      phdr->pitem->cxy, FALSE);
-      return FALSE;
-    } else if (nmhdr && nmhdr->code == HDN_ITEMCLICK) {
-      gtk_signal_emit(GTK_OBJECT(widget), "click-column", (gint)phdr->iItem);
-      return FALSE;
+    if (nmhdr) {
+      switch(nmhdr->code) {
+      case HDN_ENDTRACKA:
+        phdr = (HD_NOTIFYA FAR *)lParam;
+        gtk_clist_set_column_width_full(GTK_CLIST(widget), phdr->iItem,
+                                        phdr->pitem->cxy, FALSE);
+        return FALSE;
+      case HDN_ENDTRACKW:
+        phdrw = (HD_NOTIFYW FAR *)lParam;
+        gtk_clist_set_column_width_full(GTK_CLIST(widget), phdrw->iItem,
+                                        phdrw->pitem->cxy, FALSE);
+        return FALSE;
+      case HDN_ITEMCLICKA:
+        phdr = (HD_NOTIFYA FAR *)lParam;
+        gtk_signal_emit(GTK_OBJECT(widget), "click-column", (gint)phdr->iItem);
+        return FALSE;
+      case HDN_ITEMCLICKW:
+        phdrw = (HD_NOTIFYW FAR *)lParam;
+        gtk_signal_emit(GTK_OBJECT(widget), "click-column", (gint)phdrw->iItem);
+        return FALSE;
+      default:
+        break;
+      }
     }
     break;
   }
@@ -107,18 +164,39 @@ gboolean gtk_clist_wndproc(GtkWidget *widget, UINT msg, WPARAM wParam,
   return FALSE;
 }
 
+static void gtk_clist_set_extent(GtkCList *clist)
+{
+  gint i;
+  HWND hWnd;
+
+  hWnd = GTK_WIDGET(clist)->hWnd;
+  if (hWnd) {
+    int width = 0;
+
+    for (i = 0; i < clist->cols; i++) {
+      width += clist->coldata[i].width;
+    }
+    mySendMessage(hWnd, LB_SETHORIZONTALEXTENT, (WPARAM)width, 0);
+    SetCListHeaderSize(clist);
+  }
+}
+
 void gtk_clist_set_size(GtkWidget *widget, GtkAllocation *allocation)
 {
   GtkCList *clist = GTK_CLIST(widget);
 
   gtk_container_set_size(widget, allocation);
   if (clist->header) {
-    SetWindowPos(clist->header, HWND_TOP,
-                 allocation->x, allocation->y,
+    POINT pt;
+    pt.x = allocation->x;
+    pt.y = allocation->y;
+    MapWidgetOrigin(widget, &pt);
+    SetWindowPos(clist->scrollwin, HWND_TOP, pt.x, pt.y,
                  allocation->width, clist->header_size, SWP_NOZORDER);
     allocation->y += clist->header_size - 1;
     allocation->height -= clist->header_size - 1;
   }
+  gtk_clist_set_extent(clist);
 }
 
 GtkWidget *gtk_clist_new(gint columns)
@@ -127,14 +205,17 @@ GtkWidget *gtk_clist_new(gint columns)
   int i;
 
   clist = GTK_CLIST(GtkNewObject(&GtkCListClass));
+  clist->scrollpos = 0;
   clist->cols = columns;
   clist->coldata = g_new0(GtkCListColumn, columns);
   clist->rows = 0;
   clist->rowdata = NULL;
   for (i = 0; i < columns; i++) {
     clist->coldata[i].width = 0;
+    clist->coldata[i].optimal_width = 0;
     clist->coldata[i].visible = TRUE;
     clist->coldata[i].resizeable = TRUE;
+    clist->coldata[i].justification = GTK_JUSTIFY_LEFT;
   }
 
   return GTK_WIDGET(clist);
@@ -152,7 +233,7 @@ void gtk_clist_size_request(GtkWidget *widget, GtkRequisition *requisition)
 
 void gtk_clist_realize(GtkWidget *widget)
 {
-  HWND Parent, header;
+  HWND Parent, header, scrollwin;
   HD_LAYOUT hdl;
   HD_ITEM hdi;
   RECT rcParent;
@@ -167,33 +248,44 @@ void gtk_clist_realize(GtkWidget *widget)
   GTK_WIDGET_SET_FLAGS(widget, GTK_CAN_FOCUS);
   rcParent.left = rcParent.top = 0;
   rcParent.right = rcParent.bottom = 800;
-  header = CreateWindowEx(0, WC_HEADER, NULL,
-                          WS_CHILD | WS_BORDER | HDS_HORZ
-                          | (GTK_CLIST(widget)->coldata[0].button_passive ?
-                             0 : HDS_BUTTONS),
-                          0, 0, 0, 0, Parent, NULL, hInst, NULL);
+  scrollwin = myCreateWindow(WC_GTKCLISTHDR, NULL, WS_CHILD | WS_BORDER,
+                             0, 0, 0, 0, Parent, NULL, hInst, NULL);
+  SetWindowLong(scrollwin, GWL_USERDATA, (LONG)widget);
+  header = myCreateWindowEx(0, WC_HEADER, NULL,
+                            WS_CHILD | HDS_HORZ | WS_VISIBLE
+                            | (GTK_CLIST(widget)->coldata[0].button_passive ?
+                               0 : HDS_BUTTONS),
+                            0, 0, 0, 0, scrollwin, NULL, hInst, NULL);
   SetWindowLong(header, GWL_USERDATA, (LONG)widget);
   GTK_CLIST(widget)->header = header;
+  GTK_CLIST(widget)->scrollwin = scrollwin;
   gtk_set_default_font(header);
   hdl.prc = &rcParent;
   hdl.pwpos = &wp;
-  SendMessage(header, HDM_LAYOUT, 0, (LPARAM)&hdl);
+  mySendMessage(header, HDM_LAYOUT, 0, (LPARAM)&hdl);
   clist->header_size = wp.cy;
-  widget->hWnd = CreateWindowEx(WS_EX_CLIENTEDGE, "LISTBOX", "",
-                                WS_CHILD | WS_TABSTOP | LBS_DISABLENOSCROLL
-                                | WS_VSCROLL | LBS_OWNERDRAWFIXED |
-                                LBS_NOTIFY, 0, 0, 0, 0, Parent, NULL,
-                                hInst, NULL);
+  widget->hWnd = myCreateWindowEx(WS_EX_CLIENTEDGE, "LISTBOX", "",
+                                  WS_CHILD | WS_TABSTOP | WS_VSCROLL
+                                  | WS_HSCROLL | LBS_OWNERDRAWFIXED
+                                  | LBS_NOTIFY, 0, 0, 0, 0, Parent, NULL,
+                                  hInst, NULL);
+  /* Subclass the window */
+  wpOrigListProc = (WNDPROC)mySetWindowLong(widget->hWnd,
+                                            GWL_WNDPROC, (LONG)ListWndProc);
   gtk_set_default_font(widget->hWnd);
 
-  gtk_clist_update_all_widths(clist);
   for (rows = clist->rowdata; rows; rows = g_slist_next(rows)) {
     row = (GtkCListRow *)rows->data;
-    if (row)
-      SendMessage(widget->hWnd, LB_ADDSTRING, 0, (LPARAM)row->data);
+    if (row) {
+      mySendMessage(widget->hWnd, LB_ADDSTRING, 0, 1);
+    }
   }
+  gtk_clist_update_all_widths(clist);
 
   for (i = 0; i < clist->cols; i++) {
+    if (clist->coldata[i].auto_resize) {
+      clist->coldata[i].width = clist->coldata[i].optimal_width;
+    }
     hdi.mask = HDI_TEXT | HDI_FORMAT | HDI_WIDTH;
     hdi.pszText = clist->coldata[i].title;
     if (hdi.pszText) {
@@ -203,7 +295,7 @@ void gtk_clist_realize(GtkWidget *widget)
         hdi.cxy = clist->coldata[i].width;
       hdi.cchTextMax = strlen(hdi.pszText);
       hdi.fmt = HDF_LEFT | HDF_STRING;
-      SendMessage(header, HDM_INSERTITEM, i + 1, (LPARAM)&hdi);
+      myHeader_InsertItem(header, i + 1, &hdi);
     }
   }
 }
@@ -211,14 +303,14 @@ void gtk_clist_realize(GtkWidget *widget)
 void gtk_clist_show(GtkWidget *widget)
 {
   if (GTK_WIDGET_REALIZED(widget)) {
-    ShowWindow(GTK_CLIST(widget)->header, SW_SHOWNORMAL);
+    ShowWindow(GTK_CLIST(widget)->scrollwin, SW_SHOWNORMAL);
   }
 }
 
 void gtk_clist_hide(GtkWidget *widget)
 {
   if (GTK_WIDGET_REALIZED(widget)) {
-    ShowWindow(GTK_CLIST(widget)->header, SW_HIDE);
+    ShowWindow(GTK_CLIST(widget)->scrollwin, SW_HIDE);
   }
 }
 
@@ -227,7 +319,8 @@ void gtk_clist_draw_row(GtkCList *clist, LPDRAWITEMSTRUCT lpdis)
   HBRUSH bkgrnd;
   COLORREF textcol, oldtextcol;
   RECT rcCol;
-  gint i, CurrentX;
+  int oldbkmode;
+  gint i, CurrentX, right;
   GtkCListRow *row;
 
   if (lpdis->itemState & ODS_SELECTED) {
@@ -238,12 +331,17 @@ void gtk_clist_draw_row(GtkCList *clist, LPDRAWITEMSTRUCT lpdis)
     textcol = (COLORREF)GetSysColor(COLOR_WINDOWTEXT);
   }
   oldtextcol = SetTextColor(lpdis->hDC, textcol);
-  SetBkMode(lpdis->hDC, TRANSPARENT);
+  oldbkmode = SetBkMode(lpdis->hDC, TRANSPARENT);
   FillRect(lpdis->hDC, &lpdis->rcItem, bkgrnd);
 
   if (lpdis->itemID >= 0 && lpdis->itemID < clist->rows) {
+    int width;
     row = (GtkCListRow *)g_slist_nth_data(clist->rowdata, lpdis->itemID);
-    CurrentX = lpdis->rcItem.left;
+    width = CurrentX = 0;
+    for (i = 0; i < clist->cols; i++) {
+      width += clist->coldata[i].width;
+    }
+    right = MAX(lpdis->rcItem.right, width);
     rcCol.top = lpdis->rcItem.top;
     rcCol.bottom = lpdis->rcItem.bottom;
     if (row->text)
@@ -251,23 +349,36 @@ void gtk_clist_draw_row(GtkCList *clist, LPDRAWITEMSTRUCT lpdis)
         rcCol.left = CurrentX + LISTITEMHPACK;
         CurrentX += clist->coldata[i].width;
         rcCol.right = CurrentX - LISTITEMHPACK;
-        if (rcCol.left > lpdis->rcItem.right)
-          rcCol.left = lpdis->rcItem.right;
-        if (rcCol.right > lpdis->rcItem.right)
-          rcCol.right = lpdis->rcItem.right;
+        if (rcCol.left > right)
+          rcCol.left = right;
+        if (rcCol.right > right - LISTITEMHPACK)
+          rcCol.right = right - LISTITEMHPACK;
         if (i == clist->cols - 1)
-          rcCol.right = lpdis->rcItem.right;
+          rcCol.right = right - LISTITEMHPACK;
         if (row->text[i]) {
-          DrawText(lpdis->hDC, row->text[i], -1, &rcCol,
-                   DT_LEFT | DT_SINGLELINE | DT_VCENTER | DT_END_ELLIPSIS);
+          UINT align;
+          switch(clist->coldata[i].justification) {
+          case GTK_JUSTIFY_RIGHT:
+            align = DT_RIGHT;
+            break;
+          case GTK_JUSTIFY_CENTER:
+            align = DT_CENTER;
+            break;
+          default:
+            align = DT_LEFT;
+            break;
+          }
+          myDrawText(lpdis->hDC, row->text[i], -1, &rcCol,
+                     align | DT_SINGLELINE | DT_VCENTER | DT_END_ELLIPSIS);
         }
       }
   }
 
   SetTextColor(lpdis->hDC, oldtextcol);
-  SetBkMode(lpdis->hDC, OPAQUE);
-  if (lpdis->itemState & ODS_FOCUS)
+  SetBkMode(lpdis->hDC, oldbkmode);
+  if (lpdis->itemState & ODS_FOCUS) {
     DrawFocusRect(lpdis->hDC, &lpdis->rcItem);
+  }
 }
 
 void gtk_clist_do_auto_resize(GtkCList *clist)
@@ -276,32 +387,43 @@ void gtk_clist_do_auto_resize(GtkCList *clist)
 
   for (i = 0; i < clist->cols; i++)
     if (clist->coldata[i].auto_resize) {
-      gtk_clist_set_column_width(clist, i, clist->coldata[i].width);
+      gtk_clist_set_column_width(clist, i, clist->coldata[i].optimal_width);
     }
 }
 
+gint gtk_clist_optimal_column_width(GtkCList *clist, gint column)
+{
+  return clist->coldata[column].optimal_width;
+}
+
 void gtk_clist_update_all_widths(GtkCList *clist)
 {
   GSList *list;
   GtkCListRow *row;
-  gint i;
   SIZE size;
   HWND header;
+  gint i;
 
   header = clist->header;
-  if (header)
+  if (header) {
     for (i = 0; i < clist->cols; i++) {
-      if (GetTextSize(header, clist->coldata[i].title, &size, defFont) &&
-          clist->coldata[i].width < size.cx + 2 * LISTHEADERPACK) {
-        clist->coldata[i].width = size.cx + 2 * LISTHEADERPACK;
+      if (GetTextSize(header, clist->coldata[i].title, &size, defFont)) {
+        int new_width = size.cx + 4 + 2 * LISTHEADERPACK;
+        clist->coldata[i].width = MAX(clist->coldata[i].width, new_width);
+        clist->coldata[i].optimal_width = MAX(clist->coldata[i].optimal_width,
+                                              new_width);
       }
     }
+  }
 
   for (list = clist->rowdata; list; list = g_slist_next(list)) {
     row = (GtkCListRow *)list->data;
-    if (row && row->text)
+    if (row && row->text) {
       gtk_clist_update_widths(clist, row->text);
+    }
   }
+
+  gtk_clist_set_extent(clist);
 }
 
 void gtk_clist_update_widths(GtkCList *clist, gchar *text[])
@@ -314,10 +436,10 @@ void gtk_clist_update_widths(GtkCList *clist, gchar *text[])
   if (!hWnd)
     return;
   for (i = 0; i < clist->cols; i++) {
-    if (clist->coldata[i].auto_resize
-        && GetTextSize(hWnd, text[i], &size, defFont)
-        && size.cx + 2 * LISTITEMHPACK > clist->coldata[i].width) {
-      clist->coldata[i].width = size.cx + 2 * LISTITEMHPACK;
+    if (GetTextSize(hWnd, text[i], &size, defFont)) {
+      int new_width = size.cx + 4 + 2 * LISTITEMHPACK;
+      clist->coldata[i].optimal_width = MAX(clist->coldata[i].optimal_width,
+                                            new_width);
     }
   }
 }
@@ -334,6 +456,7 @@ gint gtk_clist_insert(GtkCList *clist, gint row, gchar *text[])
 
   new_row = g_new0(GtkCListRow, 1);
   new_row->text = g_new0(gchar *, clist->cols);
+  new_row->data = GINT_TO_POINTER(1);
 
   for (i = 0; i < clist->cols; i++) {
     new_row->text[i] = g_strdup(text[i]);
@@ -345,7 +468,7 @@ gint gtk_clist_insert(GtkCList *clist, gint row, gchar *text[])
 
   if (GTK_WIDGET_REALIZED(widget)) {
     hWnd = widget->hWnd;
-    SendMessage(hWnd, LB_INSERTSTRING, (WPARAM)row, (LPARAM)NULL);
+    mySendMessage(hWnd, LB_INSERTSTRING, (WPARAM)row, 1);
   }
 
   return row;
@@ -393,7 +516,7 @@ void gtk_clist_remove(GtkCList *clist, gint row)
     if (GTK_WIDGET_REALIZED(GTK_WIDGET(clist))) {
       HWND hWnd = GTK_WIDGET(clist)->hWnd;
 
-      SendMessage(hWnd, LB_DELETESTRING, (WPARAM)row, 0);
+      mySendMessage(hWnd, LB_DELETESTRING, (WPARAM)row, 0);
     }
   }
 }
@@ -496,13 +619,14 @@ void gtk_clist_set_column_width_full(GtkCList *clist, gint column,
       if (column == clist->cols - 1)
         width = 9000;
       hdi.cxy = width;
-      if (SendMessage(header, HDM_GETITEM, (WPARAM)column, (LPARAM)&hdi) &&
-          hdi.cxy != width) {
+      if (mySendMessage(header, HDM_GETITEM, (WPARAM)column, (LPARAM)&hdi)
+          && hdi.cxy != width) {
         hdi.mask = HDI_WIDTH;
         hdi.cxy = width;
-        SendMessage(header, HDM_SETITEM, (WPARAM)column, (LPARAM)&hdi);
+        mySendMessage(header, HDM_SETITEM, (WPARAM)column, (LPARAM)&hdi);
       }
     }
+    gtk_clist_set_extent(clist);
     hWnd = GTK_WIDGET(clist)->hWnd;
     if (hWnd)
       InvalidateRect(hWnd, NULL, FALSE);
@@ -550,14 +674,14 @@ void gtk_clist_sort(GtkCList *clist)
           rowind = GPOINTER_TO_INT(sel->data);
         else
           rowind = -1;
-        SendMessage(hWnd, LB_SETCURSEL, (WPARAM)rowind, 0);
+        mySendMessage(hWnd, LB_SETCURSEL, (WPARAM)rowind, 0);
       } else {
         for (rowind = 0; rowind < clist->rows; rowind++) {
-          SendMessage(hWnd, LB_SETSEL, (WPARAM)FALSE, (LPARAM)rowind);
+          mySendMessage(hWnd, LB_SETSEL, (WPARAM)FALSE, (LPARAM)rowind);
         }
         for (sel = clist->selection; sel; sel = g_list_next(sel)) {
           rowind = GPOINTER_TO_INT(sel->data);
-          SendMessage(hWnd, LB_SETSEL, (WPARAM)TRUE, (LPARAM)rowind);
+          mySendMessage(hWnd, LB_SETSEL, (WPARAM)TRUE, (LPARAM)rowind);
         }
       }
       InvalidateRect(hWnd, NULL, FALSE);
@@ -595,7 +719,7 @@ void gtk_clist_clear(GtkCList *clist)
   gtk_clist_update_all_widths(clist);
   hWnd = GTK_WIDGET(clist)->hWnd;
   if (hWnd) {
-    SendMessage(hWnd, LB_RESETCONTENT, 0, 0);
+    mySendMessage(hWnd, LB_RESETCONTENT, 0, 0);
   }
 }
 
@@ -638,9 +762,9 @@ void gtk_clist_select_row(GtkCList *clist, gint row, gint column)
   hWnd = GTK_WIDGET(clist)->hWnd;
   if (hWnd) {
     if (clist->mode == GTK_SELECTION_SINGLE) {
-      SendMessage(hWnd, LB_SETCURSEL, (WPARAM)row, 0);
+      mySendMessage(hWnd, LB_SETCURSEL, (WPARAM)row, 0);
     } else {
-      SendMessage(hWnd, LB_SETSEL, (WPARAM)TRUE, (LPARAM)row);
+      mySendMessage(hWnd, LB_SETSEL, (WPARAM)TRUE, (LPARAM)row);
     }
     gtk_clist_update_selection(GTK_WIDGET(clist));
   }
@@ -653,9 +777,9 @@ void gtk_clist_unselect_row(GtkCList *clist, gint row, gint column)
   hWnd = GTK_WIDGET(clist)->hWnd;
   if (hWnd) {
     if (clist->mode == GTK_SELECTION_SINGLE) {
-      SendMessage(hWnd, LB_SETCURSEL, (WPARAM)(-1), 0);
+      mySendMessage(hWnd, LB_SETCURSEL, (WPARAM)(-1), 0);
     } else {
-      SendMessage(hWnd, LB_SETSEL, (WPARAM)FALSE, (LPARAM)row);
+      mySendMessage(hWnd, LB_SETSEL, (WPARAM)FALSE, (LPARAM)row);
     }
     gtk_clist_update_selection(GTK_WIDGET(clist));
   }
@@ -696,7 +820,7 @@ void gtk_clist_update_selection(GtkWidget *widget)
   clist->selection = NULL;
   if (widget->hWnd) {
     for (i = 0; i < clist->rows; i++) {
-      if (SendMessage(widget->hWnd, LB_GETSEL, (WPARAM)i, 0) > 0) {
+      if (mySendMessage(widget->hWnd, LB_GETSEL, (WPARAM)i, 0) > 0) {
         clist->selection = g_list_append(clist->selection, GINT_TO_POINTER(i));
       }
     }
@@ -720,6 +844,48 @@ void gtk_clist_update_selection(GtkWidget *widget)
   g_list_free(oldsel);
 }
 
+void gtk_clist_set_column_justification(GtkCList *clist, gint column,
+                                        GtkJustification justification)
+{
+  clist->coldata[column].justification = justification;
+}
+
+static LRESULT CALLBACK CListHdrWndProc(HWND hwnd, UINT msg, WPARAM wParam,
+                                        LPARAM lParam)
+{
+  GtkWidget *widget;
+  gboolean retval = FALSE, dodef = TRUE;
+
+  widget = GTK_WIDGET(GetWindowLong(hwnd, GWL_USERDATA));
+
+  if (widget) {
+    retval = gtk_clist_wndproc(widget, msg, wParam, lParam, &dodef);
+  }
+
+  if (dodef) {
+    return myDefWindowProc(hwnd, msg, wParam, lParam);
+  } else {
+    return retval;
+  }
+}
+
+void InitCListClass(HINSTANCE hInstance)
+{
+  WNDCLASS wc;
+
+  wc.style = 0;
+  wc.lpfnWndProc = CListHdrWndProc;
+  wc.cbClsExtra = 0;
+  wc.cbWndExtra = 0;
+  wc.hInstance = hInstance;
+  wc.hIcon = NULL;
+  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
+  wc.hbrBackground = NULL;
+  wc.lpszMenuName = NULL;
+  wc.lpszClassName = WC_GTKCLISTHDR;
+  myRegisterClass(&wc);
+}
+
 #else /* for systems with GTK+ */
 
 GtkWidget *gtk_scrolled_clist_new_with_titles(gint columns,
