@@ -1,6 +1,6 @@
 /************************************************************************
  * gtkport.h      Portable "almost-GTK+" for Unix/Win32                 *
- * Copyright (C)  1998-2002  Ben Webb                                   *
+ * Copyright (C)  1998-2003  Ben Webb                                   *
  *                Email: ben@bellatrix.pcl.ox.ac.uk                     *
  *                WWW: http://dopewars.sourceforge.net/                 *
  *                                                                      *
@@ -35,6 +35,8 @@
 #include <glib.h>
 #include <stdarg.h>
 
+#include "gtkenums.h"
+
 #define MB_IMMRETURN 0
 
 #define MYWM_SOCKETDATA (WM_USER+100)
@@ -45,71 +47,6 @@
 
 extern HICON mainIcon;
 
-typedef enum {
-  GTK_WINDOW_TOPLEVEL, GTK_WINDOW_DIALOG, GTK_WINDOW_POPUP
-} GtkWindowType;
-
-typedef enum {
-  GTK_ACCEL_VISIBLE        = 1 << 0,
-  GTK_ACCEL_SIGNAL_VISIBLE = 1 << 1
-} GtkAccelFlags;
-
-typedef enum {
-  GTK_BUTTONBOX_SPREAD,
-  GTK_BUTTONBOX_EDGE,
-  GTK_BUTTONBOX_START,
-  GTK_BUTTONBOX_END
-} GtkButtonBoxStyle;
-
-typedef enum {
-  GTK_STATE_NORMAL,
-  GTK_STATE_ACTIVE,
-  GTK_STATE_PRELIGHT,
-  GTK_STATE_SELECTED,
-  GTK_STATE_INSENSITIVE
-} GtkStateType;
-
-typedef enum {
-  GTK_VISIBILITY_NONE,
-  GTK_VISIBILITY_PARTIAL,
-  GTK_VISIBILITY_FULL
-} GtkVisibility;
-
-typedef enum {
-  GTK_PROGRESS_LEFT_TO_RIGHT,
-  GTK_PROGRESS_RIGHT_TO_LEFT,
-  GTK_PROGRESS_BOTTOM_TO_TOP,
-  GTK_PROGRESS_TOP_TO_BOTTOM
-} GtkProgressBarOrientation;
-
-typedef enum {
-  GTK_EXPAND = 1 << 0,
-  GTK_SHRINK = 1 << 1,
-  GTK_FILL   = 1 << 2
-} GtkAttachOptions;
-
-typedef enum {
-  GTK_SELECTION_SINGLE,
-  GTK_SELECTION_BROWSE,
-  GTK_SELECTION_MULTIPLE,
-  GTK_SELECTION_EXTENDED
-} GtkSelectionMode;
-
-typedef enum {
-  GDK_INPUT_READ      = 1 << 0,
-  GDK_INPUT_WRITE     = 1 << 1,
-  GDK_INPUT_EXCEPTION = 1 << 2
-} GdkInputCondition;
-
-typedef enum
-{
-  GTK_SHADOW_NONE,
-  GTK_SHADOW_IN,
-  GTK_SHADOW_OUT,
-  GTK_SHADOW_ETCHED_IN,
-  GTK_SHADOW_ETCHED_OUT
-} GtkShadowType;
-
 #define GDK_KP_0 0xFFB0
 #define GDK_KP_1 0xFFB1
 #define GDK_KP_2 0xFFB2
@@ -127,16 +64,6 @@ typedef void (*GdkInputFunction) (gpointer data, gint source,
 typedef gchar *(*GtkTranslateFunc) (const gchar *path, gpointer func_data);
 typedef void (*GtkDestroyNotify) (gpointer data);
 
-typedef enum {
-  GTK_REALIZED    = 1 << 6,
-  GTK_VISIBLE     = 1 << 8,
-  GTK_SENSITIVE   = 1 << 10,
-  GTK_CAN_FOCUS   = 1 << 11,
-  GTK_HAS_FOCUS   = 1 << 12,
-  GTK_CAN_DEFAULT = 1 << 13,
-  GTK_IS_DEFAULT  = 1 << 14
-} GtkWidgetFlags;
-
 #define GTK_VISIBLE 1
 
 typedef struct _GtkClass GtkClass;
@@ -275,6 +202,8 @@ struct _GtkMenuItem {
   gint ID;
   gint accelind;
   gchar *text;
+  gint check:1;
+  gint active:1;
 };
 
 struct _GtkMenuShell {
@@ -296,6 +225,7 @@ struct _GtkMenuBar {
 typedef struct _GtkEditable GtkEditable;
 typedef struct _GtkEntry GtkEntry;
 typedef struct _GtkText GtkText;
+typedef struct _GtkTextBuffer GtkTextBuffer;
 typedef struct _GtkSpinButton GtkSpinButton;
 
 struct _GtkEditable {
@@ -315,9 +245,14 @@ struct _GtkSpinButton {
   HWND updown;
 };
 
+struct _GtkTextBuffer {
+  GData *tags;
+};
+
 struct _GtkText {
   GtkEditable editable;
   gint word_wrap:1;
+  GtkTextBuffer *buffer;
 };
 
 typedef struct _GtkLabel GtkLabel;
@@ -408,6 +343,7 @@ struct _GtkVBox {
 
 struct _GtkNotebookChild {
   GtkWidget *child, *tab_label;
+  HWND tabpage;
 };
 
 struct _GtkNotebook {
@@ -534,6 +470,7 @@ extern HINSTANCE hInst;
 #define GTK_MENU_SHELL(obj) ((GtkMenuShell *)(obj))
 #define GTK_MENU_BAR(obj) ((GtkMenuBar *)(obj))
 #define GTK_MENU_ITEM(obj) ((GtkMenuItem *)(obj))
+#define GTK_CHECK_MENU_ITEM(obj) ((GtkMenuItem *)(obj))
 #define GTK_MENU(obj) ((GtkMenu *)(obj))
 #define GTK_MISC(obj) ((GtkMisc *)(obj))
 #define GTK_PROGRESS_BAR(obj) ((GtkProgressBar *)(obj))
@@ -617,6 +554,8 @@ void gtk_text_set_editable(GtkText *text, gboolean is_editable);
 void gtk_text_set_word_wrap(GtkText *text, gboolean word_wrap);
 void gtk_text_freeze(GtkText *text);
 void gtk_text_thaw(GtkText *text);
+GtkTextBuffer *gtk_text_view_get_buffer(GtkText *text);
+void gtk_text_buffer_create_tag(GtkTextBuffer *buffer, const gchar *name, ...);
 void gtk_table_attach(GtkTable *table, GtkWidget *widget,
                       guint left_attach, guint right_attach,
                       guint top_attach, guint bottom_attach,
@@ -663,6 +602,7 @@ void gtk_menu_append(GtkMenu *menu, GtkWidget *child);
 void gtk_menu_prepend(GtkMenu *menu, GtkWidget *child);
 GtkWidget *gtk_menu_item_new_with_label(const gchar *label);
 void gtk_menu_item_set_submenu(GtkMenuItem *menu_item, GtkWidget *submenu);
+void gtk_check_menu_item_set_active(GtkMenuItem *menu_item, gboolean active);
 void gtk_menu_set_active(GtkMenu *menu, guint index);
 GtkWidget *gtk_notebook_new();
 void gtk_notebook_append_page(GtkNotebook *notebook, GtkWidget *child,
@@ -670,6 +610,7 @@ void gtk_notebook_append_page(GtkNotebook *notebook, GtkWidget *child,
 void gtk_notebook_insert_page(GtkNotebook *notebook, GtkWidget *child,
                               GtkWidget *tab_label, gint position);
 void gtk_notebook_set_page(GtkNotebook *notebook, gint page_num);
+gint gtk_notebook_get_current_page(GtkNotebook *notebook);
 GtkObject *gtk_adjustment_new(gfloat value, gfloat lower, gfloat upper,
                               gfloat step_increment, gfloat page_increment,
                               gfloat page_size);
@@ -751,6 +692,8 @@ void gtk_set_default_font(HWND hWnd);
 HWND gtk_get_parent_hwnd(GtkWidget *widget);
 GtkStyle *gtk_style_new(void);
 void gtk_widget_set_style(GtkWidget *widget, GtkStyle *style);
+void gtk_window_set_type_hint(GtkWindow *window, GdkWindowTypeHint hint);
+void gtk_window_set_position(GtkWindow *window, GtkWindowPosition position);
 
 /* Functions for handling emitted signals */
 void gtk_marshal_BOOL__GPOIN(GtkObject *object, GSList *actions,
@@ -777,6 +720,7 @@ void gtk_marshal_VOID__GINT_GINT_EVENT(GtkObject *object, GSList *actions,
 
 /* Private functions */
 void gtk_container_set_size(GtkWidget *widget, GtkAllocation *allocation);
+void MapWidgetOrigin(GtkWidget *widget, POINT *pt);
 
 #else /* CYGWIN */
 
@@ -823,7 +767,8 @@ struct _GtkUrl {
 
 #if CYGWIN || !HAVE_GLIB2
 extern const gchar *GTK_STOCK_OK, *GTK_STOCK_CLOSE, *GTK_STOCK_CANCEL, 
-                   *GTK_STOCK_REFRESH, *GTK_STOCK_YES, *GTK_STOCK_NO;
+                   *GTK_STOCK_REFRESH, *GTK_STOCK_YES, *GTK_STOCK_NO,
+                   *GTK_STOCK_HELP;
 
 typedef enum
 {
@@ -856,5 +801,9 @@ void TextViewClear(GtkTextView *textview);
 GtkWidget *gtk_url_new(const gchar *text, const gchar *target,
                        const gchar *bin);
 GtkWidget *NewStockButton(const gchar *label, GtkAccelGroup *accel_group);
+gchar *GtkGetFile(const GtkWidget *parent, const gchar *oldname,
+                  const gchar *title);
+void DisplayHTML(GtkWidget *parent, const gchar *bin, const gchar *target);
+gboolean HaveUnicodeSupport(void);
 
 #endif /* __GTKPORT_H__ */
