@@ -1,6 +1,6 @@
 /************************************************************************
  * newgamedia.c   New game dialog                                       *
- * Copyright (C)  1998-2002  Ben Webb                                   *
+ * Copyright (C)  1998-2003  Ben Webb                                   *
  *                Email: ben@bellatrix.pcl.ox.ac.uk                     *
  *                WWW: http://dopewars.sourceforge.net/                 *
  *                                                                      *
@@ -36,7 +36,8 @@
 #include "newgamedia.h"
 
 struct StartGameStruct {
-  GtkWidget *dialog, *name, *hostname, *port, *antique, *status, *metaserv;
+  GtkWidget *dialog, *name, *hostname, *port, *antique, *status,
+            *metaserv, *notebook;
   Player *play;
 #ifdef NETWORKING
   HttpConnection *MetaConn;
@@ -154,7 +155,7 @@ static void DoConnect(void)
 
   oldstatus = NetBuf->status;
   oldsocks = NetBuf->sockstat;
-  if (StartNetworkBufferConnect(NetBuf, ServerName, Port)) {
+  if (StartNetworkBufferConnect(NetBuf, NULL, ServerName, Port)) {
     DisplayConnectStatus(FALSE, oldstatus, oldsocks);
     SetNetworkBufferUserPasswdFunc(NetBuf, SocksAuthDialog, NULL);
     SetNetworkBufferCallBack(NetBuf, stgam.sockstat, NULL);
@@ -185,7 +186,7 @@ static void FillMetaServerList(gboolean UseNewList)
   ServerData *ThisServer;
   gchar *titles[5];
   GSList *ListPt;
-  gint row;
+  gint row, width;
 
   if (UseNewList && !stgam.NewMetaList)
     return;
@@ -222,6 +223,14 @@ static void FillMetaServerList(gboolean UseNewList)
     if (ThisServer->CurPlayers != -1)
       g_free(titles[3]);
   }
+  if (MetaList) {
+    width = gtk_clist_optimal_column_width(GTK_CLIST(metaserv), 4);
+    gtk_clist_set_column_width(GTK_CLIST(metaserv), 4, width);
+    width = gtk_clist_optimal_column_width(GTK_CLIST(metaserv), 3);
+    gtk_clist_set_column_width(GTK_CLIST(metaserv), 3, width);
+    width = gtk_clist_optimal_column_width(GTK_CLIST(metaserv), 0);
+    gtk_clist_set_column_width(GTK_CLIST(metaserv), 0, width);
+  }
   gtk_clist_thaw(GTK_CLIST(metaserv));
 }
 
@@ -248,16 +257,23 @@ void DisplayConnectStatus(gboolean meta,
   case NBS_SOCKSCONNECT:
     switch (sockstat) {
     case NBSS_METHODS:
+      /* Tell the user that we've successfully connected to a SOCKS server,
+       * and are now ready to tell it to initiate the "real" connection */
       text = g_strdup_printf(_("Status: Connected to SOCKS server %s..."),
                              Socks.name);
       SetStartGameStatus(text);
       g_free(text);
       break;
     case NBSS_USERPASSWD:
+      /* Tell the user that the SOCKS server is asking us for a username
+       * and password */
       SetStartGameStatus(_("Status: Authenticating with SOCKS server"));
       break;
     case NBSS_CONNECT:
       text =
+      /* Tell the user that all necessary SOCKS authentication has been
+       * completed, and now we're going to try to have it connect to
+       * the final destination */
           g_strdup_printf(_("Status: Asking SOCKS for connect to %s..."),
                           meta ? MetaServer.Name : ServerName);
       SetStartGameStatus(text);
@@ -409,6 +425,21 @@ static void CloseNewGameDia(GtkWidget *widget, gpointer data)
   }
   ClearServerList(&stgam.NewMetaList);
 #endif
+
+  /* Remember which tab we chose for the next time we use this dialog */
+  NewGameType = gtk_notebook_get_current_page(GTK_NOTEBOOK(stgam.notebook));
+}
+
+static void metalist_row_select(GtkWidget *clist, gint row, gint column,
+                                GdkEvent *event, GtkWidget *conn_button)
+{
+  gtk_widget_set_sensitive(conn_button, TRUE);
+}
+
+static void metalist_row_unselect(GtkWidget *clist, gint row, gint column,
+                                  GdkEvent *event, GtkWidget *conn_button)
+{
+  gtk_widget_set_sensitive(conn_button, FALSE);
 }
 
 #ifdef NETWORKING
@@ -454,6 +485,7 @@ void NewGameDialog(Player *play)
 
   /* Title of 'New Game' dialog */
   gtk_window_set_title(GTK_WINDOW(dialog), _("New Game"));
+  my_set_dialog_position(GTK_WINDOW(dialog));
   gtk_container_set_border_width(GTK_CONTAINER(dialog), 7);
   gtk_window_add_accel_group(GTK_WINDOW(dialog), accel_group);
 
@@ -476,7 +508,7 @@ void NewGameDialog(Player *play)
 
   gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
 
-  notebook = gtk_notebook_new();
+  notebook = stgam.notebook = gtk_notebook_new();
 
 #ifdef NETWORKING
   frame = gtk_frame_new(_("Server"));
@@ -587,6 +619,11 @@ void NewGameDialog(Player *play)
   SetAccelerator(button, _("_Connect"), button, "clicked", accel_group, TRUE);
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
                      GTK_SIGNAL_FUNC(MetaServerConnect), NULL);
+  gtk_widget_set_sensitive(button, FALSE);
+  gtk_signal_connect(GTK_OBJECT(clist), "select_row",
+                     GTK_SIGNAL_FUNC(metalist_row_select), button);
+  gtk_signal_connect(GTK_OBJECT(clist), "unselect_row",
+                     GTK_SIGNAL_FUNC(metalist_row_unselect), button);
   gtk_box_pack_start_defaults(GTK_BOX(hbbox), button);
 
   gtk_box_pack_start(GTK_BOX(vbox2), hbbox, FALSE, FALSE, 0);
@@ -682,6 +719,7 @@ void AuthDialog(HttpConnection *conn, gboolean proxy, gchar *realm,
     /* Title of dialog for authenticating with a web server */
     gtk_window_set_title(GTK_WINDOW(window), _("Authentication Required"));
   }
+  my_set_dialog_position(GTK_WINDOW(window));
 
   gtk_window_set_modal(GTK_WINDOW(window), TRUE);
   gtk_window_set_transient_for(GTK_WINDOW(window),
@@ -735,7 +773,7 @@ void AuthDialog(HttpConnection *conn, gboolean proxy, gchar *realm,
   button = NewStockButton(GTK_STOCK_CANCEL, accel_group);
   gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                             GTK_SIGNAL_FUNC(gtk_widget_destroy),
-                            (gpointer)window);
+                            GTK_OBJECT(window));
   gtk_box_pack_start_defaults(GTK_BOX(hbbox), button);
 
   gtk_box_pack_start(GTK_BOX(vbox), hbbox, TRUE, TRUE, 0);
@@ -796,6 +834,7 @@ static void RealSocksAuthDialog(NetworkBuffer *netbuf, gboolean meta,
   /* Title of dialog for authenticating with a SOCKS server */
   gtk_window_set_title(GTK_WINDOW(window),
                        _("SOCKS Authentication Required"));
+  my_set_dialog_position(GTK_WINDOW(window));
 
   gtk_window_set_modal(GTK_WINDOW(window), TRUE);
   gtk_window_set_transient_for(GTK_WINDOW(window),
@@ -843,7 +882,7 @@ static void RealSocksAuthDialog(NetworkBuffer *netbuf, gboolean meta,
   button = NewStockButton(GTK_STOCK_CANCEL, accel_group);
   gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                             GTK_SIGNAL_FUNC(gtk_widget_destroy),
-                            (gpointer)window);
+                            GTK_OBJECT(window));
   gtk_box_pack_start_defaults(GTK_BOX(hbbox), button);
 
   gtk_box_pack_start(GTK_BOX(vbox), hbbox, TRUE, TRUE, 0);
