@@ -1,6 +1,6 @@
 /************************************************************************
  * optdialog.c    Configuration file editing dialog                     *
- * Copyright (C)  2002  Ben Webb                                        *
+ * Copyright (C)  2002-2003  Ben Webb                                   *
  *                Email: ben@bellatrix.pcl.ox.ac.uk                     *
  *                WWW: http://dopewars.sourceforge.net/                 *
  *                                                                      *
@@ -31,6 +31,7 @@
 #include "dopewars.h"           /* For struct GLOBALS etc. */
 #include "gtk_client.h"         /* For mainwindow etc. */
 #include "nls.h"                /* For _ function */
+#include "sound.h"              /* For SoundPlay */
 #include "gtkport/gtkport.h"    /* For gtk_ functions */
 
 struct ConfigWidget {
@@ -269,13 +270,13 @@ static void AddStructConfig(GtkWidget *table, int row, gchar *structname,
   if (gvar->BoolVal) {
     GtkWidget *check;
 
-    check = gtk_check_button_new_with_label(member->label);
+    check = gtk_check_button_new_with_label(_(member->label));
     gtk_table_attach_defaults(GTK_TABLE(table), check, 0, 2, row, row + 1);
     AddConfigWidget(check, ind);
   } else {
     GtkWidget *label, *entry;
 
-    label = gtk_label_new(member->label);
+    label = gtk_label_new(_(member->label));
     gtk_table_attach(GTK_TABLE(table), label, 0, 1, row, row + 1,
                      GTK_SHRINK, GTK_SHRINK, 0, 0);
     entry = gtk_entry_new();
@@ -488,13 +489,105 @@ static void list_row_unselect(GtkCList *clist, gint row, gint column,
   }
 }
 
+static void sound_row_select(GtkCList *clist, gint row, gint column,
+                             GdkEvent *event, gpointer data)
+{
+  GtkWidget *entry;
+  int globind;
+  gchar **text;
+
+  entry = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(clist), "entry"));
+  globind = GPOINTER_TO_INT(gtk_clist_get_row_data(clist, row));
+  g_assert(globind >=0 && globind < NUMGLOB);
+
+  text = GetGlobalString(globind, 0);
+  gtk_entry_set_text(GTK_ENTRY(entry), *text);
+}
+
+static void sound_row_unselect(GtkCList *clist, gint row, gint column,
+                               GdkEvent *event, gpointer data)
+{
+  GtkWidget *entry;
+  int globind;
+  gchar *text, **oldtext;
+
+  entry = GTK_WIDGET(gtk_object_get_data(GTK_OBJECT(clist), "entry"));
+  globind = GPOINTER_TO_INT(gtk_clist_get_row_data(clist, row));
+  g_assert(globind >=0 && globind < NUMGLOB);
+
+  text = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
+  oldtext = GetGlobalString(globind, 0);
+  g_assert(text && oldtext);
+  if (strcmp(text, *oldtext) != 0) {
+    AssignName(GetGlobalString(globind, 0), text);
+    Globals[globind].Modified = TRUE;
+  }
+  gtk_entry_set_text(GTK_ENTRY(entry), "");
+  g_free(text);
+}
+
+static void BrowseSound(GtkWidget *entry)
+{
+  gchar *oldtext, *newtext;
+  GtkWidget *dialog = gtk_widget_get_ancestor(entry, GTK_TYPE_WINDOW);
+
+  oldtext = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
+
+  newtext = GtkGetFile(dialog, oldtext, _("Select sound file"));
+  g_free(oldtext);
+  if (newtext) {
+    gtk_entry_set_text(GTK_ENTRY(entry), newtext);
+    g_free(newtext);
+  }
+}
+
+static void TestPlaySound(GtkWidget *entry)
+{
+  gchar *text;
+  gboolean sound_enabled;
+
+  text = gtk_editable_get_chars(GTK_EDITABLE(entry), 0, -1);
+  sound_enabled = IsSoundEnabled();
+  SoundEnable(TRUE);
+  SoundPlay(text);
+  SoundEnable(sound_enabled);
+  g_free(text);
+}
+
 static void OKCallback(GtkWidget *widget, GtkWidget *dialog)
 {
+  GtkToggleButton *unicode_check;
+
   SaveConfigWidgets();
-  UpdateConfigFile(NULL);
+  unicode_check = GTK_TOGGLE_BUTTON(gtk_object_get_data(GTK_OBJECT(dialog),
+                                                        "unicode_check"));
+  UpdateConfigFile(NULL, gtk_toggle_button_get_active(unicode_check));
   gtk_widget_destroy(dialog);
 }
 
+static gchar *GetHelpPage(const gchar *pagename)
+{
+  gchar *root, *file;
+
+  root = GetDocRoot();
+  file = g_strdup_printf("%shelp%c%s.html", root, G_DIR_SEPARATOR, pagename);
+  g_free(root);
+  return file;
+}
+
+static void HelpCallback(GtkWidget *widget, GtkWidget *notebook)
+{
+  const static gchar *pagehelp[] = {
+    "general", "locations", "drugs", "guns", "cops", "server", "sounds"
+  };
+  gint page = gtk_notebook_get_current_page(GTK_NOTEBOOK(notebook));
+  gchar *help;
+
+  help = GetHelpPage(pagehelp[page]);
+  DisplayHTML(widget, WebBrowser, help);
+  g_free(help);
+}
+
 static void FinishOptDialog(GtkWidget *widget, gpointer data)
 {
   FreeConfigWidgets();
@@ -510,7 +603,7 @@ static GtkWidget *CreateList(gchar *structname, struct ConfigMembers *members)
   int ind, minlistlength = 0;
   gint i, row, nummembers;
   struct GLOBALS *gvar;
-  struct ConfigMembers namemember = { "Name", "Name" };
+  struct ConfigMembers namemember = { N_("Name"), "Name" };
 
   ind = GetGlobalIndex(structname, "Name");
   g_assert(ind >= 0);
@@ -603,39 +696,62 @@ static GtkWidget *CreateList(gchar *structname, struct ConfigMembers *members)
   return hbox;
 }
 
+static void FillSoundsList(GtkCList *clist)
+{
+  gchar *rowtext[2];
+  gint i, row;
+
+  gtk_clist_freeze(clist);
+  gtk_clist_clear(clist);
+  for (i = 0; i < NUMGLOB; i++) {
+    if (strlen(Globals[i].Name) > 7
+	&& strncmp(Globals[i].Name, "Sounds.", 7) == 0) {
+      rowtext[0] = &Globals[i].Name[7];
+      rowtext[1] = _(Globals[i].Help);
+      row = gtk_clist_append(clist, rowtext);
+      gtk_clist_set_row_data(clist, row, GINT_TO_POINTER(i));
+    }
+  }
+
+  gtk_clist_thaw(clist);
+}
+
 void OptDialog(GtkWidget *widget, gpointer data)
 {
   GtkWidget *dialog, *notebook, *table, *label, *check, *entry;
-  GtkWidget *hbox, *vbox, *hsep, *button, *hbbox;
+  GtkWidget *hbox, *vbox, *vbox2, *hsep, *button, *hbbox, *clist;
+  GtkWidget *scrollwin;
   GtkAccelGroup *accel_group;
+  gchar *sound_titles[2];
+  int width;
 
   struct ConfigMembers locmembers[] = {
-    { "Police presence", "PolicePresence" },
-    { "Minimum no. of drugs", "MinDrug" },
-    { "Maximum no. of drugs", "MaxDrug" },
+    { N_("Police presence"), "PolicePresence" },
+    { N_("Minimum no. of drugs"), "MinDrug" },
+    { N_("Maximum no. of drugs"), "MaxDrug" },
     { NULL, NULL }
   };
   struct ConfigMembers drugmembers[] = {
-    { "Minimum normal price", "MinPrice" },
-    { "Maximum normal price", "MaxPrice" },
-    { "Can be specially cheap", "Cheap" },
-    { "Cheap string", "CheapStr" },
-    { "Can be specially expensive", "Expensive" },
+    { N_("Minimum normal price"), "MinPrice" },
+    { N_("Maximum normal price"), "MaxPrice" },
+    { N_("Can be specially cheap"), "Cheap" },
+    { N_("Cheap string"), "CheapStr" },
+    { N_("Can be specially expensive"), "Expensive" },
     { NULL, NULL }
   };
   struct ConfigMembers gunmembers[] = {
-    { "Price", "Price" },
-    { "Inventory space", "Space" },
-    { "Damage", "Damage" },
+    { N_("Price"), "Price" },
+    { N_("Inventory space"), "Space" },
+    { N_("Damage"), "Damage" },
     { NULL, NULL }
   };
   struct ConfigMembers copmembers[] = {
-    { "Name of one deputy", "DeputyName" },
-    { "Name of several deputies", "DeputiesName" },
-    { "Minimum no. of deputies", "MinDeputies" },
-    { "Maximum no. of deputies", "MaxDeputies" },
-    { "Cop armour", "Armour" },
-    { "Deputy armour", "DeputyArmour" },
+    { N_("Name of one deputy"), "DeputyName" },
+    { N_("Name of several deputies"), "DeputiesName" },
+    { N_("Minimum no. of deputies"), "MinDeputies" },
+    { N_("Maximum no. of deputies"), "MaxDeputies" },
+    { N_("Cop armour"), "Armour" },
+    { N_("Deputy armour"), "DeputyArmour" },
     { NULL, NULL }
   };
 
@@ -644,6 +760,7 @@ void OptDialog(GtkWidget *widget, gpointer data)
   gtk_window_add_accel_group(GTK_WINDOW(dialog), accel_group);
 
   gtk_window_set_title(GTK_WINDOW(dialog), _("Options"));
+  my_set_dialog_position(GTK_WINDOW(dialog));
   gtk_container_set_border_width(GTK_CONTAINER(dialog), 7);
   gtk_window_set_modal(GTK_WINDOW(dialog), TRUE);
 
@@ -654,16 +771,18 @@ void OptDialog(GtkWidget *widget, gpointer data)
 
   notebook = gtk_notebook_new();
 
-  table = gtk_table_new(7, 3, FALSE);
+  table = gtk_table_new(8, 3, FALSE);
   gtk_table_set_row_spacings(GTK_TABLE(table), 5);
   gtk_table_set_col_spacings(GTK_TABLE(table), 5);
 
   check = NewConfigCheck("Sanitized", _("Remove drug references"));
   gtk_table_attach_defaults(GTK_TABLE(table), check, 0, 1, 0, 1);
 
-#ifdef CYGWIN
-  check = NewConfigCheck("MinToSysTray", _("Put server in System Tray"));
-  gtk_table_attach_defaults(GTK_TABLE(table), check, 1, 2, 0, 1);
+#ifdef HAVE_GLIB2
+  check = gtk_check_button_new_with_label(_("Unicode config file"));
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check), IsConfigFileUTF8());
+  gtk_table_attach_defaults(GTK_TABLE(table), check, 1, 3, 0, 1);
+  gtk_object_set_data(GTK_OBJECT(dialog), "unicode_check", check);
 #endif
 
   label = gtk_label_new(_("Game length (turns)"));
@@ -704,6 +823,14 @@ void OptDialog(GtkWidget *widget, gpointer data)
   entry = NewConfigEntry("Names.Bitches");
   gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 3, 6, 7);
 
+#ifndef CYGWIN
+  label = gtk_label_new(_("Web browser"));
+  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 7, 8,
+                   GTK_SHRINK, GTK_SHRINK, 0, 0);
+  entry = NewConfigEntry("WebBrowser");
+  gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 3, 7, 8);
+#endif
+
   gtk_container_set_border_width(GTK_CONTAINER(table), 7);
   label = gtk_label_new(_("General"));
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table, label);
@@ -714,31 +841,33 @@ void OptDialog(GtkWidget *widget, gpointer data)
   label = gtk_label_new(_("Locations"));
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook), hbox, label);
 
-  table = gtk_table_new(4, 2, FALSE);
-  gtk_table_set_row_spacings(GTK_TABLE(table), 5);
-  gtk_table_set_col_spacings(GTK_TABLE(table), 5);
-  gtk_container_set_border_width(GTK_CONTAINER(table), 7);
+  vbox2 = gtk_vbox_new(FALSE, 8);
+  gtk_container_set_border_width(GTK_CONTAINER(vbox2), 7);
 
   hbox = CreateList("Drug", drugmembers);
-  gtk_table_attach_defaults(GTK_TABLE(table), hbox, 0, 2, 0, 1);
+  gtk_box_pack_start(GTK_BOX(vbox2), hbox, TRUE, TRUE, 0);
 
   hsep = gtk_hseparator_new();
-  gtk_table_attach_defaults(GTK_TABLE(table), hsep, 0, 2, 1, 2);
+  gtk_box_pack_start(GTK_BOX(vbox2), hsep, FALSE, FALSE, 0);
 
+  table = gtk_table_new(2, 2, FALSE);
+  gtk_table_set_row_spacings(GTK_TABLE(table), 5);
+  gtk_table_set_col_spacings(GTK_TABLE(table), 5);
   label = gtk_label_new(_("Expensive string 1"));
-  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 2, 3,
+  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 0, 1,
                    GTK_SHRINK, GTK_SHRINK, 0, 0);
   entry = NewConfigEntry("Drugs.ExpensiveStr1");
-  gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 2, 3);
+  gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 0, 1);
 
   label = gtk_label_new(_("Expensive string 2"));
-  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 3, 4,
+  gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
                    GTK_SHRINK, GTK_SHRINK, 0, 0);
   entry = NewConfigEntry("Drugs.ExpensiveStr2");
-  gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 3, 4);
+  gtk_table_attach_defaults(GTK_TABLE(table), entry, 1, 2, 1, 2);
+  gtk_box_pack_start(GTK_BOX(vbox2), table, FALSE, FALSE, 0);
 
   label = gtk_label_new(_("Drugs"));
-  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table, label);
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox2, label);
 
   hbox = CreateList("Gun", gunmembers);
   gtk_container_set_border_width(GTK_CONTAINER(hbox), 7);
@@ -756,7 +885,12 @@ void OptDialog(GtkWidget *widget, gpointer data)
 
   check = NewConfigCheck("MetaServer.Active",
                          _("Server reports to metaserver"));
-  gtk_table_attach_defaults(GTK_TABLE(table), check, 0, 4, 0, 1);
+  gtk_table_attach_defaults(GTK_TABLE(table), check, 0, 2, 0, 1);
+
+#ifdef CYGWIN
+  check = NewConfigCheck("MinToSysTray", _("Minimize to System Tray"));
+  gtk_table_attach_defaults(GTK_TABLE(table), check, 2, 4, 0, 1);
+#endif
 
   label = gtk_label_new(_("Metaserver hostname"));
   gtk_table_attach(GTK_TABLE(table), label, 0, 1, 1, 2,
@@ -804,6 +938,47 @@ void OptDialog(GtkWidget *widget, gpointer data)
   label = gtk_label_new(_("Server"));
   gtk_notebook_append_page(GTK_NOTEBOOK(notebook), table, label);
 
+  vbox2 = gtk_vbox_new(FALSE, 5);
+  gtk_container_set_border_width(GTK_CONTAINER(vbox2), 7);
+
+  sound_titles[0] = _("Sound name");
+  sound_titles[1] = _("Description");
+  clist = gtk_scrolled_clist_new_with_titles(2, sound_titles, &scrollwin);
+  gtk_clist_column_titles_passive(GTK_CLIST(clist));
+  gtk_clist_set_selection_mode(GTK_CLIST(clist), GTK_SELECTION_SINGLE);
+  FillSoundsList(GTK_CLIST(clist));
+  gtk_signal_connect(GTK_OBJECT(clist), "select_row",
+                     GTK_SIGNAL_FUNC(sound_row_select), NULL);
+  gtk_signal_connect(GTK_OBJECT(clist), "unselect_row",
+                     GTK_SIGNAL_FUNC(sound_row_unselect), NULL);
+
+  clists = g_slist_append(clists, clist);
+
+  gtk_box_pack_start(GTK_BOX(vbox2), scrollwin, TRUE, TRUE, 0);
+
+  hbox = gtk_hbox_new(FALSE, 5);
+  label = gtk_label_new(_("Sound file"));
+  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
+
+  entry = gtk_entry_new();
+  gtk_object_set_data(GTK_OBJECT(clist), "entry", entry);
+  gtk_box_pack_start(GTK_BOX(hbox), entry, TRUE, TRUE, 0);
+
+  button = gtk_button_new_with_label(_("Browse..."));
+  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
+                            GTK_SIGNAL_FUNC(BrowseSound), GTK_OBJECT(entry));
+  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+
+  button = gtk_button_new_with_label(_("Play"));
+  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
+                            GTK_SIGNAL_FUNC(TestPlaySound), GTK_OBJECT(entry));
+  gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
+
+  gtk_box_pack_start(GTK_BOX(vbox2), hbox, FALSE, FALSE, 0);
+
+  label = gtk_label_new(_("Sounds"));
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), vbox2, label);
+
   gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 0);
 
   gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
@@ -817,10 +992,16 @@ void OptDialog(GtkWidget *widget, gpointer data)
   gtk_signal_connect(GTK_OBJECT(button), "clicked",
                      GTK_SIGNAL_FUNC(OKCallback), (gpointer)dialog);
   gtk_box_pack_start_defaults(GTK_BOX(hbbox), button);
+
+  button = NewStockButton(GTK_STOCK_HELP, accel_group);
+  gtk_signal_connect(GTK_OBJECT(button), "clicked",
+                     GTK_SIGNAL_FUNC(HelpCallback), (gpointer)notebook);
+  gtk_box_pack_start_defaults(GTK_BOX(hbbox), button);
+
   button = NewStockButton(GTK_STOCK_CANCEL, accel_group);
   gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                             GTK_SIGNAL_FUNC(gtk_widget_destroy),
-                            (gpointer)dialog);
+                            GTK_OBJECT(dialog));
   gtk_signal_connect(GTK_OBJECT(dialog), "destroy",
                      GTK_SIGNAL_FUNC(FinishOptDialog), NULL);
   gtk_box_pack_start_defaults(GTK_BOX(hbbox), button);
@@ -830,4 +1011,7 @@ void OptDialog(GtkWidget *widget, gpointer data)
   gtk_container_add(GTK_CONTAINER(dialog), vbox);
 
   gtk_widget_show_all(dialog);
+
+  width = gtk_clist_optimal_column_width(GTK_CLIST(clist), 0);
+  gtk_clist_set_column_width(GTK_CLIST(clist), 0, width);
 }
