@@ -1,6 +1,6 @@
 /************************************************************************
  * message.c      Message-handling routines for dopewars                *
- * Copyright (C)  1998-2002  Ben Webb                                   *
+ * Copyright (C)  1998-2003  Ben Webb                                   *
  *                Email: ben@bellatrix.pcl.ox.ac.uk                     *
  *                WWW: http://dopewars.sourceforge.net/                 *
  *                                                                      *
@@ -36,6 +36,8 @@
 #include <string.h>
 #include <stdlib.h>
 #include <glib.h>
+
+#include "convert.h"
 #include "dopewars.h"
 #include "message.h"
 #include "network.h"
@@ -98,7 +100,9 @@
    and vice versa.  */
 /* *INDENT-ON* */
 
-GSList *FirstClient;
+GSList *FirstClient = NULL;
+
+static Converter *netconv = NULL;
 
 void (*ClientMessageHandlerPt)(char *, Player *) = NULL;
 
@@ -247,6 +251,12 @@ void InitAbilities(Player *Play)
   Play->Abil.Local[A_DRUGVALUE] = (DrugValue ? TRUE : FALSE);
   Play->Abil.Local[A_TSTRING] = TRUE;
   Play->Abil.Local[A_DONEFIGHT] = TRUE;
+#ifdef HAVE_GLIB2
+  Play->Abil.Local[A_UTF8] = TRUE;
+#else
+  Play->Abil.Local[A_UTF8] = FALSE;
+#endif
+  Play->Abil.Local[A_DATE] = TRUE;
 
   if (!Network) {
     for (i = 0; i < A_NUM; i++) {
@@ -309,6 +319,10 @@ void CombineAbilities(Player *Play)
   for (i = 0; i < A_NUM; i++) {
     Play->Abil.Shared[i] = (Play->Abil.Remote[i] && Play->Abil.Local[i]);
   }
+
+  if (HaveAbility(Play, A_UTF8)) {
+    Conv_SetCodeset(netconv, "UTF-8");
+  }
 }
 
 /* 
@@ -361,7 +375,16 @@ gboolean PlayerHandleNetwork(Player *Play, gboolean ReadReady,
 
 gchar *GetWaitingPlayerMessage(Player *Play)
 {
-  return GetWaitingMessage(&Play->NetBuf);
+  gchar *unconv, *conv;
+
+  unconv = GetWaitingMessage(&Play->NetBuf);
+  if (unconv && Conv_Needed(netconv)) {
+    conv = Conv_ToInternal(netconv, unconv, -1);
+    g_free(unconv);
+    return conv;
+  } else {
+    return unconv;
+  }
 }
 
 gboolean ReadPlayerDataFromWire(Player *Play)
@@ -371,7 +394,13 @@ gboolean ReadPlayerDataFromWire(Player *Play)
 
 void QueuePlayerMessageForSend(Player *Play, gchar *data)
 {
-  QueueMessageForSend(&Play->NetBuf, data);
+  if (Conv_Needed(netconv)) {
+    gchar *conv = Conv_ToExternal(netconv, data, -1);
+    QueueMessageForSend(&Play->NetBuf, conv);
+    g_free(conv);
+  } else {
+    QueueMessageForSend(&Play->NetBuf, data);
+  }
 }
 
 gboolean WritePlayerDataToWire(Player *Play)
@@ -413,6 +442,7 @@ gboolean OpenMetaHttpConnection(HttpConnection **conn)
                           MetaServer.Path, METAVERSION);
   retval = OpenHttpConnection(conn, MetaServer.Name, MetaServer.Port,
                               MetaServer.ProxyName, MetaServer.ProxyPort,
+                              "",
                               UseSocks
                               && MetaServer.UseSocks ? &Socks : NULL,
                               "GET", query, NULL, NULL);
@@ -624,6 +654,10 @@ void SendSpyReport(Player *To, Player *SpiedOn)
   g_free(cashstr);
   g_free(debtstr);
   g_free(bankstr);
+  if (HaveAbility(SpiedOn, A_DATE)) {
+    g_string_sprintfa(text, "%d^%d^%d^", g_date_day(SpiedOn->date),
+                      g_date_month(SpiedOn->date), g_date_year(SpiedOn->date));
+  }
   for (i = 0; i < NumGun; i++) {
     g_string_sprintfa(text, "%d^", SpiedOn->Guns[i].Carried);
   }
@@ -645,13 +679,13 @@ void SendSpyReport(Player *To, Player *SpiedOn)
   g_string_free(text, TRUE);
 }
 
-#define NUMNAMES 12
+#define NUMNAMES 11
 
 void SendInitialData(Player *To)
 {
   gchar *LocalNames[NUMNAMES] = { Names.Bitch, Names.Bitches, Names.Gun,
     Names.Guns, Names.Drug, Names.Drugs,
-    Names.Month, Names.Year, Names.LoanSharkName,
+    Names.Date, Names.LoanSharkName,
     Names.BankName, Names.GunShopName,
     Names.RoughPubName
   };
@@ -667,15 +701,23 @@ void SendInitialData(Player *To)
   text = g_string_new("");
   g_string_sprintf(text, "%s^%d^%d^%d^", VERSION, NumLocation, NumGun,
                    NumDrug);
-  for (i = 0; i < 8; i++) {
+  for (i = 0; i < 6; i++) {
     g_string_append(text, LocalNames[i]);
     g_string_append_c(text, '^');
   }
+
+  if (HaveAbility(To, A_DATE)) {
+    g_string_append(text, LocalNames[6]);
+    g_string_append_c(text, '^');
+  } else {
+    g_string_sprintfa(text, "%d-^-%d^", StartDate.month, StartDate.year);
+  }
+
   if (HaveAbility(To, A_PLAYERID))
     g_string_sprintfa(text, "%d^", To->ID);
 
-  /* Player ID is expected after the first 8 names, so send the rest now */
-  for (i = 8; i < NUMNAMES; i++) {
+  /* Player ID is expected after the first 7 names, so send the rest now */
+  for (i = 7; i < NUMNAMES; i++) {
     g_string_append(text, LocalNames[i]);
     g_string_append_c(text, '^');
   }
@@ -710,8 +752,17 @@ void ReceiveInitialData(Player *Play, char *Data)
   AssignName(&Names.Guns, GetNextWord(&pt, ""));
   AssignName(&Names.Drug, GetNextWord(&pt, ""));
   AssignName(&Names.Drugs, GetNextWord(&pt, ""));
-  AssignName(&Names.Month, GetNextWord(&pt, ""));
-  AssignName(&Names.Year, GetNextWord(&pt, ""));
+  if (HaveAbility(Play, A_DATE)) {
+    AssignName(&Names.Date, GetNextWord(&pt, ""));
+  } else {
+    gchar *month, *year, *date;
+    month = GetNextWord(&pt, "");
+    year = GetNextWord(&pt, "");
+
+    date = g_strdup_printf("%s%%T%s", month, year);
+    AssignName(&Names.Date, date);
+    g_free(date);
+  }
   if (HaveAbility(Play, A_PLAYERID))
     Play->ID = GetNextInt(&pt, 0);
 
@@ -851,6 +902,11 @@ void ReceivePlayerData(Player *Play, char *text, Player *From)
   From->IsAt = GetNextInt(&cp, 0);
   From->Turn = GetNextInt(&cp, 0);
   From->Flags = GetNextInt(&cp, 0);
+  if (HaveAbility(Play, A_DATE)) {
+    g_date_set_day(From->date, GetNextInt(&cp, 1));
+    g_date_set_month(From->date, GetNextInt(&cp, 1));
+    g_date_set_year(From->date, GetNextInt(&cp, 1980));
+  }
   for (i = 0; i < NumGun; i++) {
     From->Guns[i].Carried = GetNextInt(&cp, 0);
   }
@@ -933,6 +989,14 @@ void SwitchToSinglePlayer(Player *Play)
   }
 }
 
+void InitNetwork(void)
+{
+  netconv = Conv_New();
+#ifdef NETWORKING
+  StartNetworking();
+#endif
+}
+
 /* 
  * Closes down the client side of the network connection. Clears the list
  * of client players (with the exception of "you", the player "Play"),
@@ -1170,6 +1234,12 @@ void ReceiveFightMessage(gchar *Data, gchar **AttackName,
   switch (*fp) {
   case F_HIT:
     SoundPlay(Sounds.FightHit);
+    if (*BitchesKilled > 0) {
+      SoundPlay(*DefendName[0] ? Sounds.EnemyBitchKilled : Sounds.BitchKilled);
+    }
+    if (*DefendHealth <= 0) {
+      SoundPlay(*DefendName[0] ? Sounds.EnemyKilled : Sounds.Killed);
+    }
     break;
   case F_MISS:
     SoundPlay(Sounds.FightMiss);
@@ -1177,6 +1247,13 @@ void ReceiveFightMessage(gchar *Data, gchar **AttackName,
   case F_RELOAD:
     SoundPlay(Sounds.FightReload);
     break;
+  case F_FAILFLEE:
+    SoundPlay(*AttackName[0] ? Sounds.EnemyFailFlee : Sounds.FailFlee);
+    break;
+  case F_LEAVE:
+  case F_LASTLEAVE:
+    SoundPlay(*AttackName[0] ? Sounds.EnemyFlee : Sounds.Flee);
+    break;
   default:
     break;
   }
@@ -1330,8 +1407,8 @@ void FormatFightMessage(Player *To, GString *text, Player *Attacker,
       if (AttackName[0]) {
         if (!IsCop(Attacker) && brandom(0, 100) < 70
             && Attacker->IsAt >= 0) {
-          g_string_sprintfa(text, _("%s has got away to %s!"), AttackName,
-                            Location[Attacker->IsAt].Name);
+          dpg_string_sprintfa(text, _("%s has got away to %tde!"), AttackName,
+                              Location[Attacker->IsAt].Name);
         } else {
           g_string_sprintfa(text, _("%s has got away!"), AttackName);
         }
