@@ -1,6 +1,6 @@
 /************************************************************************
  * message.h      Header file for dopewars message-handling routines    *
- * Copyright (C)  1998-2002  Ben Webb                                   *
+ * Copyright (C)  1998-2003  Ben Webb                                   *
  *                Email: ben@bellatrix.pcl.ox.ac.uk                     *
  *                WWW: http://dopewars.sourceforge.net/                 *
  *                                                                      *
@@ -49,7 +49,8 @@ typedef enum {
   C_NONE = 'A',
   C_ASKLOAN, C_COPSMESG, C_ASKBITCH, C_ASKGUN, C_ASKGUNSHOP,
   C_ASKPUB, C_ASKBANK, C_ASKRUN, C_ASKRUNFIGHT, C_ASKSEW,
-  C_MEETPLAYER, C_FIGHT, C_FIGHTDONE, C_MOTD, C_VERSIONCHECK
+  C_MEETPLAYER, C_FIGHT, C_FIGHTDONE, C_MOTD, C_VERSIONCHECK,
+  C_MISSFIGHT
 } AICode;
 
 #define DT_LOCATION    'A'
@@ -92,6 +93,7 @@ extern GSList *FirstClient;
 
 extern void (*ClientMessageHandlerPt) (char *, Player *);
 
+void InitNetwork(void);
 void AddURLEnc(GString *str, gchar *unenc);
 void chomp(char *str);
 void BroadcastToClients(AICode AI, MsgCode Code, char *Data, Player *From,
