@@ -1,6 +1,6 @@
 /************************************************************************
  * network.c      Low-level networking routines                         *
- * Copyright (C)  1998-2002  Ben Webb                                   *
+ * Copyright (C)  1998-2003  Ben Webb                                   *
  *                Email: ben@bellatrix.pcl.ox.ac.uk                     *
  *                WWW: http://dopewars.sourceforge.net/                 *
  *                                                                      *
@@ -70,7 +70,7 @@ typedef enum {
 static gboolean StartSocksNegotiation(NetworkBuffer *NetBuf,
                                       gchar *RemoteHost,
                                       unsigned RemotePort);
-static gboolean StartConnect(int *fd, gchar *RemoteHost,
+static gboolean StartConnect(int *fd, const gchar *bindaddr, gchar *RemoteHost,
                              unsigned RemotePort, gboolean *doneOK,
                              LastError **error);
 
@@ -104,7 +104,7 @@ void StartNetworking()
           errstr->str);
     g_string_free(errstr, TRUE);
     FreeError(error);
-    exit(1);
+    exit(EXIT_FAILURE);
   }
 }
 
@@ -120,7 +120,7 @@ void SetReuse(SOCKET sock)
   if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, (char *)&i,
                  sizeof(i)) == -1) {
     perror("setsockopt");
-    exit(1);
+    exit(EXIT_FAILURE);
   }
 }
 
@@ -148,7 +148,7 @@ void SetReuse(int sock)
 
   if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &i, sizeof(i)) == -1) {
     perror("setsockopt");
-    exit(1);
+    exit(EXIT_FAILURE);
   }
 }
 
@@ -260,6 +260,7 @@ gboolean IsNetworkBufferActive(NetworkBuffer *NetBuf)
 }
 
 gboolean StartNetworkBufferConnect(NetworkBuffer *NetBuf,
+                                   const gchar *bindaddr,
                                    gchar *RemoteHost, unsigned RemotePort)
 {
   gchar *realhost;
@@ -276,7 +277,7 @@ gboolean StartNetworkBufferConnect(NetworkBuffer *NetBuf,
     realport = RemotePort;
   }
 
-  if (StartConnect(&NetBuf->fd, realhost, realport, &doneOK,
+  if (StartConnect(&NetBuf->fd, bindaddr, realhost, realport, &doneOK,
                    &NetBuf->error)) {
     /* If we connected immediately, then set status, otherwise signal that 
      * we're waiting for the connect to complete */
@@ -968,18 +969,20 @@ void QueueMessageForSend(NetworkBuffer *NetBuf, gchar *data)
   CommitWriteBuffer(NetBuf, conn, addpt, addlen);
 }
 
-static struct hostent *LookupHostname(gchar *host, LastError **error)
+static void SetNetworkError(LastError **error) {
+#ifdef CYGWIN
+  SetError(error, ET_WINSOCK, WSAGetLastError(), NULL);
+#else
+  SetError(error, ET_HERRNO, h_errno, NULL);
+#endif
+}
+
+static struct hostent *LookupHostname(const gchar *host, LastError **error)
 {
   struct hostent *he;
 
-  if ((he = gethostbyname(host)) == NULL) {
-#ifdef CYGWIN
-    if (error)
-      SetError(error, ET_WINSOCK, WSAGetLastError(), NULL);
-#else
-    if (error)
-      SetError(error, ET_HERRNO, h_errno, NULL);
-#endif
+  if ((he = gethostbyname(host)) == NULL && error) {
+    SetNetworkError(error);
   }
   return he;
 }
@@ -1201,7 +1204,8 @@ static gboolean StartHttpConnect(HttpConnection *conn)
     ConnectPort = conn->Port;
   }
 
-  if (!StartNetworkBufferConnect(&conn->NetBuf, ConnectHost, ConnectPort)) {
+  if (!StartNetworkBufferConnect(&conn->NetBuf, conn->bindaddr, ConnectHost,
+                                 ConnectPort)) {
     return FALSE;
   }
   return TRUE;
@@ -1209,9 +1213,9 @@ static gboolean StartHttpConnect(HttpConnection *conn)
 
 gboolean OpenHttpConnection(HttpConnection **connpt, gchar *HostName,
                             unsigned Port, gchar *Proxy,
-                            unsigned ProxyPort, SocksServer *socks,
-                            gchar *Method, gchar *Query, gchar *Headers,
-                            gchar *Body)
+                            unsigned ProxyPort, const gchar *bindaddr,
+                            SocksServer *socks, gchar *Method,
+                            gchar *Query, gchar *Headers, gchar *Body)
 {
   HttpConnection *conn;
 
@@ -1223,6 +1227,7 @@ gboolean OpenHttpConnection(HttpConnection **connpt, gchar *HostName,
   conn->HostName = g_strdup(HostName);
   if (Proxy && Proxy[0])
     conn->Proxy = g_strdup(Proxy);
+  conn->bindaddr = g_strdup(bindaddr);
   conn->Method = g_strdup(Method);
   conn->Query = g_strdup(Query);
   if (Headers && Headers[0])
@@ -1246,6 +1251,7 @@ void CloseHttpConnection(HttpConnection *conn)
   ShutdownNetworkBuffer(&conn->NetBuf);
   g_free(conn->HostName);
   g_free(conn->Proxy);
+  g_free(conn->bindaddr);
   g_free(conn->Method);
   g_free(conn->Query);
   g_free(conn->Headers);
@@ -1508,42 +1514,44 @@ int CreateTCPSocket(LastError **error)
   fd = socket(AF_INET, SOCK_STREAM, 0);
 
   if (fd == SOCKET_ERROR && error) {
-#ifdef CYGWIN
-    SetError(error, ET_WINSOCK, WSAGetLastError(), NULL);
-#else
-    SetError(error, ET_ERRNO, errno, NULL);
-#endif
+    SetNetworkError(error);
   }
 
   return fd;
 }
 
-gboolean BindTCPSocket(int sock, unsigned port, LastError **error)
+gboolean BindTCPSocket(int sock, const gchar *addr, unsigned port,
+                       LastError **error)
 {
   struct sockaddr_in bindaddr;
   int retval;
+  struct hostent *he;
 
   bindaddr.sin_family = AF_INET;
   bindaddr.sin_port = htons(port);
-  bindaddr.sin_addr.s_addr = INADDR_ANY;
+  if (addr && addr[0]) {
+    he = LookupHostname(addr, error);
+    if (!he) {
+      return FALSE;
+    }
+    bindaddr.sin_addr = *((struct in_addr *)he->h_addr);
+  } else {
+    bindaddr.sin_addr.s_addr = INADDR_ANY;
+  }
   memset(bindaddr.sin_zero, 0, sizeof(bindaddr.sin_zero));
 
   retval =
       bind(sock, (struct sockaddr *)&bindaddr, sizeof(struct sockaddr));
 
   if (retval == SOCKET_ERROR && error) {
-#ifdef CYGWIN
-    SetError(error, ET_WINSOCK, WSAGetLastError(), NULL);
-#else
-    SetError(error, ET_ERRNO, errno, NULL);
-#endif
+    SetNetworkError(error);
   }
 
   return (retval != SOCKET_ERROR);
 }
 
-gboolean StartConnect(int *fd, gchar *RemoteHost, unsigned RemotePort,
-                      gboolean *doneOK, LastError **error)
+gboolean StartConnect(int *fd, const gchar *bindaddr, gchar *RemoteHost,
+                      unsigned RemotePort, gboolean *doneOK, LastError **error)
 {
   struct sockaddr_in ClientAddr;
   struct hostent *he;
@@ -1558,6 +1566,10 @@ gboolean StartConnect(int *fd, gchar *RemoteHost, unsigned RemotePort,
   if (*fd == SOCKET_ERROR)
     return FALSE;
 
+  if (bindaddr && bindaddr[0] && !BindTCPSocket(*fd, bindaddr, 0, error)) {
+    return FALSE;
+  }
+
   ClientAddr.sin_family = AF_INET;
   ClientAddr.sin_port = htons(RemotePort);
   ClientAddr.sin_addr = *((struct in_addr *)he->h_addr);
