@@ -1,6 +1,6 @@
 /************************************************************************
  * sound.h        Header file for dopewars sound system                 *
- * Copyright (C)  1998-2002  Ben Webb                                   *
+ * Copyright (C)  1998-2003  Ben Webb                                   *
  *                Email: ben@bellatrix.pcl.ox.ac.uk                     *
  *                WWW: http://dopewars.sourceforge.net/                 *
  *                                                                      *
@@ -42,5 +42,7 @@ void SoundInit(void);
 void SoundOpen(gchar *drivername);
 void SoundClose(void);
 void SoundPlay(const gchar *snd);
+void SoundEnable(gboolean enable);
+gboolean IsSoundEnabled(void);
 
 #endif /* __DP_SOUND_H__ */
