@@ -1,6 +1,6 @@
 /************************************************************************
  * winmain.c      Startup code and support for the Win32 platform       *
- * Copyright (C)  1998-2002  Ben Webb                                   *
+ * Copyright (C)  1998-2003  Ben Webb                                   *
  *                Email: ben@bellatrix.pcl.ox.ac.uk                     *
  *                WWW: http://dopewars.sourceforge.net/                 *
  *                                                                      *
@@ -32,6 +32,7 @@
 #include <stdlib.h>
 
 #include "dopewars.h"
+#include "log.h"
 #include "nls.h"
 #include "tstring.h"
 #include "util.h"
@@ -178,8 +179,9 @@ static gchar *GetWindowsLocale(void)
   switch (lang) {
   case LANG_ENGLISH:
     strcat(langenv, "en");
-    if (sublang == SUBLANG_ENGLISH_UK)
-      strcat(langenv, "_gb");
+    if (sublang == SUBLANG_ENGLISH_UK) {
+      strcat(langenv, "_GB");
+    }
     break;
   case LANG_FRENCH:
     strcat(langenv, "fr");
@@ -190,6 +192,9 @@ static gchar *GetWindowsLocale(void)
   case LANG_POLISH:
     strcat(langenv, "pl");
     break;
+  case LANG_SPANISH:
+    strcat(langenv, "es");
+    break;
   case LANG_NORWEGIAN:
     if (sublang == SUBLANG_NORWEGIAN_NYNORSK) {
       strcat(langenv, "nn");
@@ -199,8 +204,9 @@ static gchar *GetWindowsLocale(void)
     break;
   case LANG_PORTUGUESE:
     strcat(langenv, "pt");
-    if (sublang == SUBLANG_PORTUGUESE_BRAZILIAN)
-      strcat(langenv, "_br");
+    if (sublang == SUBLANG_PORTUGUESE_BRAZILIAN) {
+      strcat(langenv, "_BR");
+    }
     break;
   }
 
@@ -212,13 +218,33 @@ static gchar *GetWindowsLocale(void)
 }
 #endif /* ENABLE_NLS */
 
+/*
+ * Converts the passed lpszCmdParam WinMain-style command line into
+ * Unix-style (argc, argv) parameters (returned).
+ */
+void get_cmdline(const LPSTR lpszCmdParam, int *argc, gchar ***argv)
+{
+  gchar **split;
+  gchar *tmp_cmdline;
+
+  tmp_cmdline = g_strdup_printf("%s %s", PACKAGE, lpszCmdParam);
+  split = g_strsplit(tmp_cmdline, " ", 0);
+  *argc = 0;
+  while (split[*argc] && split[*argc][0]) {
+    (*argc)++;
+  }
+  *argv = split;
+  g_free(tmp_cmdline);
+}
+
 int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                      LPSTR lpszCmdParam, int nCmdShow)
 {
-  gchar **split;
+  gchar **argv;
   int argc;
   gboolean is_service;
   gchar *modpath;
+  struct CMDLINE *cmdline;
 
 #ifdef ENABLE_NLS
   gchar *winlocale;
@@ -260,46 +286,38 @@ int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
             "# informative messages resulting from configuration\n"
             "# file processing and the like.\n\n"));
 
-#ifdef HAVE_GLIB2
-  split = g_strsplit(lpszCmdParam, " ", 1);
-#else
-  split = g_strsplit(lpszCmdParam, " ", 0);
-#endif
-  argc = 0;
-  while (split[argc] && split[argc][0])
-    argc++;
+  get_cmdline(lpszCmdParam, &argc, &argv);
 
-  SoundInit();
-  GeneralStartup(argc, split);
+  cmdline = GeneralStartup(argc, argv);
   OpenLog();
-  if (WantVersion || WantHelp) {
+  SoundInit();
+  if (cmdline->version || cmdline->help) {
     WindowPrintStart();
     g_set_print_handler(WindowPrintFunc);
-    HandleHelpTexts();
+    HandleHelpTexts(cmdline->help);
     WindowPrintEnd();
 #ifdef NETWORKING
   } else if (is_service) {
-    StartNetworking();
+    InitNetwork();
     Network = Server = TRUE;
     win32_init(hInstance, hPrevInstance, "mainicon");
-    ServiceMain();
+    ServiceMain(cmdline);
     StopNetworking();
 #endif
-  } else if (WantConvert) {
+  } else if (cmdline->convert) {
     WindowPrintStart();
     g_set_print_handler(WindowPrintFunc);
-    ConvertHighScoreFile();
+    ConvertHighScoreFile(cmdline->convertfile);
     WindowPrintEnd();
   } else {
-#ifdef NETWORKING
-    StartNetworking();
-#endif
-    if (Server) {
+    InitNetwork();
+    if (cmdline->server) {
 #ifdef NETWORKING
 #ifdef GUI_SERVER
+      Server = TRUE;
       g_log_set_handler(NULL, G_LOG_LEVEL_CRITICAL, LogMessage, NULL);
       win32_init(hInstance, hPrevInstance, "mainicon");
-      GuiServerLoop(FALSE);
+      GuiServerLoop(cmdline, FALSE);
 #else
       AllocConsole();
       SetConsoleTitle(_("dopewars server"));
@@ -308,7 +326,7 @@ int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                         G_LOG_LEVEL_WARNING, ServerLogMessage, NULL);
       g_set_print_handler(ServerPrintFunc);
       newterm(NULL, NULL, NULL);
-      ServerLoop();
+      ServerLoop(cmdline);
 #endif /* GUI_SERVER */
 #else
       WindowPrintStart();
@@ -319,7 +337,7 @@ int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                 "configure script.\n"));
       WindowPrintEnd();
 #endif /* NETWORKING */
-    } else if (AIPlayer) {
+    } else if (cmdline->ai) {
       AllocConsole();
 
       /* Title of the Windows window used for AI player output */
@@ -329,23 +347,23 @@ int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
                         LogMask() | G_LOG_LEVEL_MESSAGE |
                         G_LOG_LEVEL_WARNING, ServerLogMessage, NULL);
       g_set_print_handler(ServerPrintFunc);
-      AIPlayerLoop();
+      AIPlayerLoop(cmdline);
     } else {
-      switch (WantedClient) {
+      switch (cmdline->client) {
       case CLIENT_AUTO:
-        if (!GtkLoop(hInstance, hPrevInstance, TRUE)) {
+        if (!GtkLoop(hInstance, hPrevInstance, cmdline, TRUE)) {
           AllocConsole();
           SetConsoleTitle(_("dopewars"));
-          CursesLoop();
+          CursesLoop(cmdline);
         }
         break;
       case CLIENT_WINDOW:
-        GtkLoop(hInstance, hPrevInstance, FALSE);
+        GtkLoop(hInstance, hPrevInstance, cmdline, FALSE);
         break;
       case CLIENT_CURSES:
         AllocConsole();
         SetConsoleTitle(_("dopewars"));
-        CursesLoop();
+        CursesLoop(cmdline);
         break;
       }
     }
@@ -353,13 +371,13 @@ int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
     StopNetworking();
 #endif
   }
+  FreeCmdLine(cmdline);
   CloseLog();
   LogFileEnd();
-  g_strfreev(split);
+  g_strfreev(argv);
   CloseHighScoreFile();
   g_free(PidFile);
   g_free(Log.File);
-  g_free(ConvertFile);
   SoundClose();
   return 0;
 }
