@@ -22,7 +22,12 @@
 
 #include <windows.h>
 #include <stdio.h>
-#include "zlib/zlib.h"
+#include <stdlib.h>
+#ifdef LIBBZ2
+# include "bzlib/bzlib.h"
+#else
+# include "zlib/zlib.h"
+#endif
 #include "util.h"
 
 char *read_line(HANDLE hin)
@@ -110,7 +115,7 @@ InstData *ReadInstallData()
       switch (section) {
       case S_NONE:
         printf("Bad line %s\n", line);
-        exit(1);
+        exit(EXIT_FAILURE);
       case S_PRODUCT:
         printf("product ID = %s\n", line);
         idata->product = line;
@@ -160,12 +165,12 @@ InstData *ReadInstallData()
     return idata;
   } else {
     printf("No directories specified\n");
-    exit(1);
+    exit(EXIT_FAILURE);
   }
 }
 
 #define BUFFER_SIZE (32*1024)
-#define COMPRESSION Z_BEST_COMPRESSION
+#define COMPRESSION 9
 
 void OpenNextFile(InstFiles *filelist, InstFiles **listpt, HANDLE *fin)
 {
@@ -202,20 +207,31 @@ int main()
   char *inbuf, *outbuf;
   int status, count;
   bstr *str;
+#ifdef LIBBZ2
+  bz_stream z;
+#else
   z_stream z;
+#endif
 
   idata = ReadInstallData();
 
-  fout = CreateFile("installfiles.gz", GENERIC_WRITE, 0, NULL,
+  fout = CreateFile("installfiles.z", GENERIC_WRITE, 0, NULL,
                     CREATE_ALWAYS, 0, NULL);
 
   outbuf = bmalloc(BUFFER_SIZE);
   inbuf = bmalloc(BUFFER_SIZE);
 
+#ifdef LIBBZ2
+  z.bzalloc = NULL;
+  z.bzfree = NULL;
+  z.opaque = NULL;
+  BZ2_bzCompressInit(&z, COMPRESSION, 0, 30);
+#else
   z.zalloc = Z_NULL;
-  z.zfree = Z_NULL;
+  z.zfree  = Z_NULL;
   z.opaque = Z_NULL;
   deflateInit(&z, COMPRESSION);
+#endif
   z.avail_in = 0;
   z.next_out = outbuf;
   z.avail_out = BUFFER_SIZE;
@@ -242,27 +258,46 @@ int main()
       }
       z.avail_in = bytes_read;
     }
-    if (z.avail_in == 0) {
-      status = deflate(&z, Z_FINISH);
-      count = BUFFER_SIZE - z.avail_out;
-      if (!WriteFile(fout, outbuf, count, &bytes_written, NULL)) {
-        printf("Write error\n");
-      }
-      break;
-    }
-    status = deflate(&z, Z_NO_FLUSH);
+#ifdef LIBBZ2
+    status = BZ2_bzCompress(&z, z.avail_in == 0 ? BZ_FINISH : BZ_RUN);
+#else
+    status = deflate(&z, z.avail_in == 0 ? Z_FINISH : Z_NO_FLUSH);
+#endif
     count = BUFFER_SIZE - z.avail_out;
     if (!WriteFile(fout, outbuf, count, &bytes_written, NULL)) {
       printf("Write error\n");
     }
     z.next_out = outbuf;
     z.avail_out = BUFFER_SIZE;
+
+#ifdef LIBBZ2
+    if (status == BZ_STREAM_END) {
+      break;
+    } else if (status != BZ_RUN_OK && status != BZ_FINISH_OK) {
+      printf("Unexpected bzlib status: %d\n", status);
+      break;
+    }
+#else
+    if (status == Z_STREAM_END) {
+      break;
+    } else if (status != Z_OK) {
+      printf("Unexpected libz status: %d\n", status);
+      break;
+    }
+#endif
   }
 
-  printf("Written compressed data: raw %lu, compressed %lu\n",
+#ifdef LIBBZ2
+  printf("Written compressed data: raw %d, compressed %d\n",
+         z.total_in_lo32, z.total_out_lo32);
+  bytes_written = z.total_out_lo32;
+  BZ2_bzCompressEnd(&z);
+#else
+  printf("Written compressed data: raw %ld, compressed %ld\n",
          z.total_in, z.total_out);
   bytes_written = z.total_out;
   deflateEnd(&z);
+#endif
 
   CloseHandle(fout);
 
@@ -309,3 +344,10 @@ int main()
 
   return 0;
 }
+
+#ifdef LIBBZ2
+void bz_internal_error(int errcode)
+{
+  printf("bzip error %d\n", errcode);
+}
+#endif
