@@ -25,7 +25,11 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include "zlib/zlib.h"
+#ifdef LIBBZ2
+# include "bzlib/bzlib.h"
+#else
+# include "zlib/zlib.h"
+#endif
 #include <shlobj.h>
 
 #include "contid.h"
@@ -429,7 +433,11 @@ char *GetFirstFile(InstFiles *filelist, DWORD totalsize)
   DWORD bufsiz;
   char *inbuf, *outbuf;
   int status;
+#ifdef LIBBZ2
+  bz_stream z;
+#else
   z_stream z;
+#endif
 
   if (!filelist)
     return NULL;
@@ -441,22 +449,42 @@ char *GetFirstFile(InstFiles *filelist, DWORD totalsize)
   bufsiz = filelist->filesize;
   outbuf = bmalloc(bufsiz + 1);
 
+#ifdef LIBBZ2
+  z.bzalloc = NULL;
+  z.bzfree = NULL;
+  z.opaque = NULL;
+  BZ2_bzDecompressInit(&z, 0, 0);
+#else
   z.zalloc = Z_NULL;
-  z.zfree = Z_NULL;
+  z.zfree  = Z_NULL;
   z.opaque = Z_NULL;
+  inflateInit(&z);
+#endif
+
   z.next_in = inbuf;
   z.avail_in = totalsize;
-
-  inflateInit(&z);
   z.next_out = outbuf;
   z.avail_out = bufsiz;
 
   while (1) {
-    status = inflate(&z, Z_SYNC_FLUSH);
-    if ((status != Z_OK && status != Z_STREAM_END) || z.avail_out == 0)
+#ifdef LIBBZ2
+    status = BZ2_bzDecompress(&z);
+    if ((status != BZ_OK && status != BZ_STREAM_END) || z.avail_out == 0) {
       break;
+    }
+#else
+    status = inflate(&z, Z_SYNC_FLUSH);
+    if ((status != Z_OK && status != Z_STREAM_END) || z.avail_out == 0) {
+      break;
+    }
+#endif
   }
+
+#ifdef LIBBZ2
+  BZ2_bzDecompressEnd(&z);
+#else
   inflateEnd(&z);
+#endif
 
   outbuf[bufsiz] = '\0';
   return outbuf;
@@ -827,7 +855,11 @@ DWORD WINAPI DoInstall(LPVOID lpParam)
   BOOL skipfile, service_installed;
   char *inbuf, *outbuf;
   int status, count;
+#ifdef LIBBZ2
+  bz_stream z;
+#else
   z_stream z;
+#endif
   InstFiles *listpt;
   InstData *oldidata;
 
@@ -877,19 +909,31 @@ DWORD WINAPI DoInstall(LPVOID lpParam)
 
   outbuf = bmalloc(BUFFER_SIZE);
 
+#ifdef LIBBZ2
+  z.bzalloc = NULL;
+  z.bzfree  = NULL;
+  z.opaque  = NULL;
+  BZ2_bzDecompressInit(&z, 0, 0);
+#else
   z.zalloc = Z_NULL;
-  z.zfree = Z_NULL;
+  z.zfree  = Z_NULL;
   z.opaque = Z_NULL;
-  z.next_in = inbuf;
-  z.avail_in = idata->totalsize;
-
   inflateInit(&z);
-  z.next_out = outbuf;
+#endif
+
+  z.avail_in  = idata->totalsize;
+  z.next_in   = inbuf;
+  z.next_out  = outbuf;
   z.avail_out = BUFFER_SIZE;
 
   while (1) {
+#ifdef LIBBZ2
+    status = BZ2_bzDecompress(&z);
+    if (status == BZ_OK || status == BZ_STREAM_END) {
+#else
     status = inflate(&z, Z_SYNC_FLUSH);
     if (status == Z_OK || status == Z_STREAM_END) {
+#endif
       count = BUFFER_SIZE - z.avail_out;
       z.next_out = outbuf;
       while (count >= fileleft) {
@@ -913,11 +957,22 @@ DWORD WINAPI DoInstall(LPVOID lpParam)
       z.next_out = outbuf;
       z.avail_out = BUFFER_SIZE;
     }
-    if (status != Z_OK)
+#ifdef LIBBZ2
+    if (status != BZ_OK) {
       break;
+    }
+#else
+    if (status != Z_OK) {
+      break;
+    }
+#endif
   }
 
+#ifdef LIBBZ2
+  BZ2_bzDecompressEnd(&z);
+#else
   inflateEnd(&z);
+#endif
   if (!skipfile)
     CloseHandle(fout);
 
@@ -945,7 +1000,8 @@ DWORD WINAPI DoInstall(LPVOID lpParam)
                "you turn on your computer, see the \"Services\" application\n"
                "from Control Panel. You can also run an interactive server "
                "by using\nthe \"dopewars server\" shortcut from the desktop "
-               "and/or Start Menu.", "Service Installed", MB_OK);
+               "and/or Start Menu.", "Service Installed",
+               MB_ICONINFORMATION | MB_OK);
   }
 
   CoInitialize(NULL);
@@ -1173,3 +1229,9 @@ int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
 
   return 0;
 }
+
+#ifdef LIBBZ2
+void bz_internal_error(int errcode)
+{
+}
+#endif
